@@ -36,7 +36,7 @@
 
 #include "qgsmapsettings.h"
 #include "qgs3dmapscene.h"
-#include "qgsterrainentity_p.h"
+#include "qgsterrainentity.h"
 #include "qgscoordinatereferencesystemutils.h"
 #include "qgscoordinatereferencesystem.h"
 #include "qgswindow3dengine.h"
@@ -55,11 +55,10 @@ Qgs3DAxis::Qgs3DAxis( Qgs3DMapCanvas *canvas,
   , mCameraController( cameraCtrl )
   , mCrs( map->crs() )
 {
-  mAxisViewport = constructAxisViewport( parent3DScene );
-  mAxisViewport->setParent( mCanvas->activeFrameGraph() );
+  mViewport = constructAxisScene( parent3DScene );
+  mViewport->setParent( mCanvas->activeFrameGraph() );
 
-  mTwoDLabelViewport = constructLabelViewport( parent3DScene, QRectF( 0.0f, 0.0f, 1.0f, 1.0f ) );
-  mTwoDLabelViewport->setParent( mCanvas->activeFrameGraph() );
+  constructLabelsScene( parent3DScene );
 
   connect( cameraCtrl, &QgsCameraController::cameraChanged, this, &Qgs3DAxis::onCameraUpdate );
   connect( mCanvas, &Qgs3DMapCanvas::widthChanged, this, &Qgs3DAxis::onAxisViewportSizeUpdate );
@@ -77,16 +76,39 @@ Qgs3DAxis::~Qgs3DAxis()
 {
   delete mMenu;
   mMenu = nullptr;
+
+  // When an object (axis or cube) is not enabled. It is still present but it does not have a parent.
+  // In that case, it will never be automatically deleted. Therefore, it needs to be manually deleted.
+  // See setEnableCube() and setEnableAxis().
+  switch ( mMapSettings->get3DAxisSettings().mode() )
+  {
+    case Qgs3DAxisSettings::Mode::Crs:
+      delete mCubeRoot;
+      mCubeRoot = nullptr;
+      break;
+    case Qgs3DAxisSettings::Mode::Cube:
+      delete mAxisRoot;
+      mAxisRoot = nullptr;
+      break;
+    case Qgs3DAxisSettings::Mode::Off:
+      delete mAxisRoot;
+      mAxisRoot = nullptr;
+      delete mCubeRoot;
+      mCubeRoot = nullptr;
+      break;
+  }
 }
 
 void Qgs3DAxis::init3DObjectPicking( )
 {
+  mDefaultPickingMethod = mMapScene->engine()->renderSettings()->pickingSettings()->pickMethod();
+
   // Create screencaster to be used by EventFilter:
   //   1- Perform ray casting tests by specifying "touch" coordinates in screen space
   //   2- connect screencaster results to onTouchedByRay
   //   3- screencaster will be triggered by EventFilter
   mScreenRayCaster = new Qt3DRender::QScreenRayCaster( mAxisSceneEntity );
-  mScreenRayCaster->addLayer( mAxisSceneLayer ); // to only filter on axis objects
+  mScreenRayCaster->addLayer( mAxisObjectLayer ); // to only filter on axis objects
   mScreenRayCaster->setFilterMode( Qt3DRender::QScreenRayCaster::AcceptAllMatchingLayers );
   mScreenRayCaster->setRunMode( Qt3DRender::QAbstractRayCaster::SingleShot );
 
@@ -134,17 +156,17 @@ bool Qgs3DAxis::eventFilter( QObject *watched, QEvent *event )
     else if ( ! mIsDragging )
     {
       // limit ray caster usage to the axis viewport
-      QPointF normalizedPos( static_cast<float>( mouseEvent->pos().x() ) / mCanvas->width(),
-                             ( float )mouseEvent->pos().y() / mCanvas->height() );
+      QPointF normalizedPos( static_cast<float>( mouseEvent->pos().x() ) / static_cast<float>( mCanvas->width() ),
+                             static_cast<float>( mouseEvent->pos().y() ) / static_cast<float>( mCanvas->height() ) );
 
       if ( 2 <= QgsLogger::debugLevel() && event->type() == QEvent::MouseButtonRelease )
       {
         std::ostringstream os;
-        os << "QGS3DAxis: normalized pos: " << normalizedPos << " / viewport: " << mAxisViewport->normalizedRect();
+        os << "QGS3DAxis: normalized pos: " << normalizedPos << " / viewport: " << mViewport->normalizedRect();
         QgsDebugMsgLevel( os.str().c_str(), 2 );
       }
 
-      if ( mAxisViewport->normalizedRect().contains( normalizedPos ) )
+      if ( mViewport->normalizedRect().contains( normalizedPos ) )
       {
         mLastClickedButton = mouseEvent->button();
         mLastClickedPos = mouseEvent->pos();
@@ -152,12 +174,23 @@ bool Qgs3DAxis::eventFilter( QObject *watched, QEvent *event )
         // if casted ray from pos matches an entity, call onTouchedByRay
         mScreenRayCaster->trigger( mLastClickedPos );
       }
-
-      // when we exit the viewport, reset the mouse cursor if needed
-      else if ( mPreviousCursor != Qt::ArrowCursor && mCanvas->cursor() == Qt::ArrowCursor )
+      // exit the viewport
+      else
       {
-        mCanvas->setCursor( mPreviousCursor );
-        mPreviousCursor = Qt::ArrowCursor;
+        // reset the mouse cursor if needed
+        if ( mPreviousCursor != Qt::ArrowCursor && mCanvas->cursor() == Qt::ArrowCursor )
+        {
+          mCanvas->setCursor( mPreviousCursor );
+          mPreviousCursor = Qt::ArrowCursor;
+        }
+
+        // reset the picking settings if needed
+        if ( mMapScene->engine()->renderSettings()->pickingSettings()->pickMethod() == Qt3DRender::QPickingSettings::TrianglePicking
+             && mDefaultPickingMethod != Qt3DRender::QPickingSettings::TrianglePicking )
+        {
+          mMapScene->engine()->renderSettings()->pickingSettings()->setPickMethod( mDefaultPickingMethod );
+          QgsDebugMsgLevel( "Disabling triangle picking", 2 );
+        }
       }
 
       mIsDragging = false; // drag ends
@@ -210,6 +243,14 @@ void Qgs3DAxis::onTouchedByRay( const Qt3DRender::QAbstractRayCaster::Hits &hits
         mPreviousCursor = mCanvas->cursor();
         mCanvas->setCursor( Qt::ArrowCursor );
         QgsDebugMsgLevel( "Enabling arrow cursor", 2 );
+
+        // The cube needs triangle picking to handle click on faces.
+        if ( mMapScene->engine()->renderSettings()->pickingSettings()->pickMethod() != Qt3DRender::QPickingSettings::TrianglePicking &&
+             mCubeRoot->isEnabled() )
+        {
+          mMapScene->engine()->renderSettings()->pickingSettings()->setPickMethod( Qt3DRender::QPickingSettings::TrianglePicking );
+          QgsDebugMsgLevel( "Enabling triangle picking", 2 );
+        }
       }
     }
   }
@@ -265,7 +306,7 @@ void Qgs3DAxis::onTouchedByRay( const Qt3DRender::QAbstractRayCaster::Hits &hits
   }
 }
 
-Qt3DRender::QViewport *Qgs3DAxis::constructAxisViewport( Qt3DCore::QEntity *parent3DScene )
+Qt3DRender::QViewport *Qgs3DAxis::constructAxisScene( Qt3DCore::QEntity *parent3DScene )
 {
   Qt3DRender::QViewport *axisViewport = new Qt3DRender::QViewport;
   // parent will be set later
@@ -275,10 +316,10 @@ Qt3DRender::QViewport *Qgs3DAxis::constructAxisViewport( Qt3DCore::QEntity *pare
   mAxisSceneEntity->setParent( parent3DScene );
   mAxisSceneEntity->setObjectName( "3DAxis_SceneEntity" );
 
-  mAxisSceneLayer = new Qt3DRender::QLayer;
-  mAxisSceneLayer->setObjectName( "3DAxis_SceneLayer" );
-  mAxisSceneLayer->setParent( mAxisSceneEntity );
-  mAxisSceneLayer->setRecursive( true );
+  mAxisObjectLayer = new Qt3DRender::QLayer;
+  mAxisObjectLayer->setObjectName( "3DAxis_ObjectLayer" );
+  mAxisObjectLayer->setParent( mAxisSceneEntity );
+  mAxisObjectLayer->setRecursive( true );
 
   mAxisCamera = new Qt3DRender::QCamera;
   mAxisCamera->setParent( mAxisSceneEntity );
@@ -289,12 +330,8 @@ Qt3DRender::QViewport *Qgs3DAxis::constructAxisViewport( Qt3DCore::QEntity *pare
   mAxisCamera->setViewCenter( QVector3D( 0.0f, 0.0f, 0.0f ) );
   // position will be set later
 
-  Qt3DRender::QLayer *axisLayer = new Qt3DRender::QLayer;
-  axisLayer->setRecursive( true );
-  mAxisSceneEntity->addComponent( axisLayer );
-
   Qt3DRender::QLayerFilter *axisLayerFilter = new Qt3DRender::QLayerFilter( axisViewport );
-  axisLayerFilter->addLayer( axisLayer );
+  axisLayerFilter->addLayer( mAxisObjectLayer );
 
   Qt3DRender::QCameraSelector *axisCameraSelector = new Qt3DRender::QCameraSelector;
   axisCameraSelector->setParent( axisLayerFilter );
@@ -314,12 +351,8 @@ Qt3DRender::QViewport *Qgs3DAxis::constructAxisViewport( Qt3DCore::QEntity *pare
   return axisViewport;
 }
 
-Qt3DRender::QViewport *Qgs3DAxis::constructLabelViewport( Qt3DCore::QEntity *parent3DScene, const QRectF &parentViewportSize )
+void Qgs3DAxis::constructLabelsScene( Qt3DCore::QEntity *parent3DScene )
 {
-  Qt3DRender::QViewport *twoDViewport = new Qt3DRender::QViewport;
-  // parent will be set later
-  twoDViewport->setNormalizedRect( parentViewportSize );
-
   mTwoDLabelSceneEntity = new Qt3DCore::QEntity;
   mTwoDLabelSceneEntity->setParent( parent3DScene );
   mTwoDLabelSceneEntity->setEnabled( true );
@@ -327,10 +360,7 @@ Qt3DRender::QViewport *Qgs3DAxis::constructLabelViewport( Qt3DCore::QEntity *par
   mTwoDLabelCamera = new Qt3DRender::QCamera;
   mTwoDLabelCamera->setParent( mTwoDLabelSceneEntity );
   mTwoDLabelCamera->setProjectionType( Qt3DRender::QCameraLens::ProjectionType::OrthographicProjection );
-  mTwoDLabelCamera->lens()->setOrthographicProjection(
-    -mCanvas->width() / 2.0f, mCanvas->width() / 2.0f,
-    -mCanvas->height() / 2.0f, mCanvas->height() / 2.0f,
-    -10.0f, 100.0f );
+  // the camera lens parameters are defined by onAxisViewportSizeUpdate()
 
   mTwoDLabelCamera->setUpVector( QVector3D( 0.0f, 0.0f, 1.0f ) );
   mTwoDLabelCamera->setViewCenter( QVector3D( 0.0f, 0.0f, 0.0f ) );
@@ -338,10 +368,11 @@ Qt3DRender::QViewport *Qgs3DAxis::constructLabelViewport( Qt3DCore::QEntity *par
   mTwoDLabelCamera->setPosition( QVector3D( 0.0f, 0.0f, 100.0f ) );
 
   Qt3DRender::QLayer *twoDLayer = new Qt3DRender::QLayer;
+  twoDLayer->setObjectName( "3DAxis_LabelsLayer" );
   twoDLayer->setRecursive( true );
   mTwoDLabelSceneEntity->addComponent( twoDLayer );
 
-  Qt3DRender::QLayerFilter *twoDLayerFilter = new Qt3DRender::QLayerFilter( twoDViewport );
+  Qt3DRender::QLayerFilter *twoDLayerFilter = new Qt3DRender::QLayerFilter;
   twoDLayerFilter->addLayer( twoDLayer );
 
   Qt3DRender::QCameraSelector *twoDCameraSelector = new Qt3DRender::QCameraSelector;
@@ -358,40 +389,18 @@ Qt3DRender::QViewport *Qgs3DAxis::constructLabelViewport( Qt3DCore::QEntity *par
   Qt3DRender::QClearBuffers *clearBuffers = new Qt3DRender::QClearBuffers( sortPolicy );
   clearBuffers->setBuffers( Qt3DRender::QClearBuffers::DepthBuffer );
 
-  // cppcheck-suppress memleak
-  return twoDViewport;
+  twoDLayerFilter->setParent( mViewport );
 }
 
-QVector3D Qgs3DAxis::from3DTo2DLabelPosition( const QVector3D &sourcePos,
-    Qt3DRender::QCamera *sourceCamera, Qt3DRender::QViewport *sourceViewport,
-    Qt3DRender::QCamera *destCamera, Qt3DRender::QViewport *destViewport,
-    const QSize &destSize )
+QVector3D Qgs3DAxis::from3DTo2DLabelPosition( const QVector3D &sourcePos, Qt3DRender::QCamera *sourceCamera, Qt3DRender::QCamera *destCamera )
 {
-  QVector3D destPos = sourcePos.project( sourceCamera->viewMatrix(),
-                                         destCamera->projectionMatrix(),
-                                         QRect( 0.0f, 0.0f,
-                                             destViewport->normalizedRect().width() * destSize.width(),
-                                             destViewport->normalizedRect().height() * destSize.height() ) );
-  QPointF axisCenter = sourceViewport->normalizedRect().center();
-  QPointF labelCenter = destViewport->normalizedRect().center();
-  QVector3D viewTranslation = QVector3D( ( axisCenter - labelCenter ).x() * destSize.width(),
-                                         ( axisCenter - labelCenter ).y() * destSize.height(),
-                                         0.0f );
-  destPos -= QVector3D( labelCenter.x() * destSize.width(),
-                        labelCenter.y() * destSize.height(),
-                        0.0f );
-  destPos.setX( destPos.x() + viewTranslation.x() );
-  destPos.setY( destPos.y() - viewTranslation.y() );
-  destPos.setZ( 0.0f );
+  const int viewportWidth = static_cast<int>( std::round( mTwoDLabelCamera->lens()->right() - mTwoDLabelCamera->lens()->left() ) );
+  const int viewportHeight = static_cast<int>( std::round( mTwoDLabelCamera->lens()->top() - mTwoDLabelCamera->lens()->bottom() ) );
+  QRect viewportRect( static_cast<int>( std::round( mTwoDLabelCamera->lens()->left() ) ), static_cast<int>( std::round( mTwoDLabelCamera->lens()->bottom() ) ),
+                      viewportWidth, viewportHeight );
 
-  if ( 2 <= QgsLogger::debugLevel() )
-  {
-    std::ostringstream os;
-    os << "Qgs3DAxis::from3DTo2DLabelPosition: sourcePos: " << sourcePos.toPoint()
-       << " with translation: " << viewTranslation.toPoint()
-       << " corrected to pos: " << destPos.toPoint();
-    QgsDebugMsgLevel( os.str().c_str(), 2 );
-  }
+  QVector3D destPos = sourcePos.project( sourceCamera->viewMatrix(), destCamera->projectionMatrix(), viewportRect );
+  destPos.setZ( 0.0f );
   return destPos;
 }
 
@@ -427,12 +436,12 @@ void Qgs3DAxis::setEnableAxis( bool show )
 
 void Qgs3DAxis::createAxisScene()
 {
-  if ( mAxisRoot == nullptr || mCubeRoot == nullptr )
+  if ( !mAxisRoot || !mCubeRoot )
   {
     mAxisRoot = new Qt3DCore::QEntity;
     mAxisRoot->setParent( mAxisSceneEntity );
     mAxisRoot->setObjectName( "3DAxis_AxisRoot" );
-    mAxisRoot->addComponent( mAxisSceneLayer ); // raycaster will filter object containing this layer
+    mAxisRoot->addComponent( mAxisObjectLayer ); // raycaster will filter object containing this layer
 
     createAxis( Qt::Axis::XAxis );
     createAxis( Qt::Axis::YAxis );
@@ -441,7 +450,7 @@ void Qgs3DAxis::createAxisScene()
     mCubeRoot = new Qt3DCore::QEntity;
     mCubeRoot->setParent( mAxisSceneEntity );
     mCubeRoot->setObjectName( "3DAxis_CubeRoot" );
-    mCubeRoot->addComponent( mAxisSceneLayer ); // raycaster will filter object containing this layer
+    mCubeRoot->addComponent( mAxisObjectLayer ); // raycaster will filter object containing this layer
 
     createCube( );
   }
@@ -503,7 +512,7 @@ void Qgs3DAxis::createKeyboardShortCut()
   if ( eng )
   {
     QWidget *mapCanvas = dynamic_cast<QWidget *>( eng->parent() );
-    if ( mapCanvas == nullptr )
+    if ( !mapCanvas )
     {
       QgsLogger::warning( "Qgs3DAxis: no canvas defined!" );
     }
@@ -697,7 +706,7 @@ void Qgs3DAxis::hideMenu()
 
 void Qgs3DAxis::displayMenuAt( const QPoint &sourcePos )
 {
-  if ( mMenu == nullptr )
+  if ( !mMenu )
   {
     createMenu();
   }
@@ -792,39 +801,37 @@ void Qgs3DAxis::createCube( )
 
   // text
   QString text;
-  int fontSize = 0.75 * mFontSize;
-  float textHeight = fontSize * 1.5f;
+  const int fontSize = static_cast<int>( std::round( 0.75f * static_cast<float>( mFontSize ) ) );
+  const float textHeight = static_cast<float>( fontSize ) * 1.5f;
   float textWidth;
-  QFont f = QFontDatabase::systemFont( QFontDatabase::FixedFont );
-  f.setPointSize( fontSize );
-  f.setWeight( QFont::Weight::Black );
+  const QFont font = createFont( fontSize );
 
   {
     text = QStringLiteral( "top" );
-    textWidth = text.length() * fontSize * 0.75f;
+    textWidth = static_cast<float>( text.length() * fontSize ) * 0.75f;
     QVector3D translation = minPos + QVector3D(
                               mCylinderLength * 0.5f - textWidth / 2.0f,
                               mCylinderLength * 0.5f - textHeight / 2.0f,
                               mCylinderLength * 1.01f );
     QMatrix4x4 rotation;
-    mCubeLabels << addCubeText( text, textHeight, textWidth, f, rotation, translation );
+    mCubeLabels << addCubeText( text, textHeight, textWidth, font, rotation, translation );
   }
 
   {
     text = QStringLiteral( "btm" );
-    textWidth = text.length() * fontSize * 0.75f;
+    textWidth = static_cast<float>( text.length() * fontSize ) * 0.75f;
     QVector3D translation = minPos + QVector3D(
                               mCylinderLength * 0.5f - textWidth / 2.0f,
                               mCylinderLength * 0.5f + textHeight / 2.0f,
                               -mCylinderLength * 0.01f );
     QMatrix4x4 rotation;
     rotation.rotate( 180.0f, QVector3D( 1.0f, 0.0f, 0.0f ).normalized() );
-    mCubeLabels << addCubeText( text, textHeight, textWidth, f, rotation, translation );
+    mCubeLabels << addCubeText( text, textHeight, textWidth, font, rotation, translation );
   }
 
   {
     text = QStringLiteral( "west" );
-    textWidth = text.length() * fontSize * 0.75f;
+    textWidth = static_cast<float>( text.length() * fontSize ) * 0.75f;
     QVector3D translation = minPos + QVector3D(
                               - mCylinderLength * 0.01f,
                               mCylinderLength * 0.5f + textWidth / 2.0f,
@@ -832,12 +839,12 @@ void Qgs3DAxis::createCube( )
     QMatrix4x4 rotation;
     rotation.rotate( 90.0f, QVector3D( 0.0f, -1.0f, 0.0f ).normalized() );
     rotation.rotate( 90.0f, QVector3D( 0.0f, 0.0f, -1.0f ).normalized() );
-    mCubeLabels << addCubeText( text, textHeight, textWidth, f, rotation, translation );
+    mCubeLabels << addCubeText( text, textHeight, textWidth, font, rotation, translation );
   }
 
   {
     text = QStringLiteral( "east" );
-    textWidth = text.length() * fontSize * 0.75f;
+    textWidth = static_cast<float>( text.length() * fontSize ) * 0.75f;
     QVector3D translation = minPos + QVector3D(
                               mCylinderLength * 1.01f,
                               mCylinderLength * 0.5f - textWidth / 2.0f,
@@ -845,24 +852,24 @@ void Qgs3DAxis::createCube( )
     QMatrix4x4 rotation;
     rotation.rotate( 90.0f, QVector3D( 0.0f, 1.0f, 0.0f ).normalized() );
     rotation.rotate( 90.0f, QVector3D( 0.0f, 0.0f, 1.0f ).normalized() );
-    mCubeLabels << addCubeText( text, textHeight, textWidth, f, rotation, translation );
+    mCubeLabels << addCubeText( text, textHeight, textWidth, font, rotation, translation );
   }
 
   {
     text = QStringLiteral( "south" );
-    textWidth = text.length() * fontSize * 0.75f;
+    textWidth = static_cast<float>( text.length() * fontSize ) * 0.75f;
     QVector3D translation = minPos + QVector3D(
                               mCylinderLength * 0.5f - textWidth / 2.0f,
                               - mCylinderLength * 0.01f,
                               mCylinderLength * 0.5f - textHeight / 2.0f );
     QMatrix4x4 rotation;
     rotation.rotate( 90.0f, QVector3D( 1.0f, 0.0f, 0.0f ).normalized() );
-    mCubeLabels << addCubeText( text, textHeight, textWidth, f, rotation, translation );
+    mCubeLabels << addCubeText( text, textHeight, textWidth, font, rotation, translation );
   }
 
   {
     text = QStringLiteral( "north" );
-    textWidth = text.length() * fontSize * 0.75f;
+    textWidth = static_cast<float>( text.length() * fontSize ) * 0.75f;
     QVector3D translation = minPos + QVector3D(
                               mCylinderLength * 0.5f + textWidth / 2.0f,
                               mCylinderLength * 1.01f,
@@ -870,7 +877,7 @@ void Qgs3DAxis::createCube( )
     QMatrix4x4 rotation;
     rotation.rotate( 90.0f, QVector3D( -1.0f, 0.0f, 0.0f ).normalized() );
     rotation.rotate( 180.0f, QVector3D( 0.0f, 0.0f, 1.0f ).normalized() );
-    mCubeLabels << addCubeText( text, textHeight, textWidth, f, rotation, translation );
+    mCubeLabels << addCubeText( text, textHeight, textWidth, font, rotation, translation );
   }
 
   for ( Qt3DExtras::QText2DEntity *l : std::as_const( mCubeLabels ) )
@@ -879,11 +886,11 @@ void Qgs3DAxis::createCube( )
   }
 }
 
-Qt3DExtras::QText2DEntity *Qgs3DAxis::addCubeText( const QString &text, float textHeight, float textWidth, const QFont &f, const QMatrix4x4 &rotation, const QVector3D &translation )
+Qt3DExtras::QText2DEntity *Qgs3DAxis::addCubeText( const QString &text, float textHeight, float textWidth, const QFont &font, const QMatrix4x4 &rotation, const QVector3D &translation )
 {
   Qt3DExtras::QText2DEntity *textEntity = new Qt3DExtras::QText2DEntity;
   textEntity->setObjectName( "3DAxis_cube_label_" + text );
-  textEntity->setFont( f );
+  textEntity->setFont( font );
   textEntity->setHeight( textHeight );
   textEntity->setWidth( textWidth );
   textEntity->setColor( QColor( 192, 192, 192 ) );
@@ -915,7 +922,10 @@ void Qgs3DAxis::createAxis( Qt::Axis axisType )
     case Qt::Axis::XAxis:
       mTextX = new Qt3DExtras::QText2DEntity( );  // object initialization in two step:
       mTextX->setParent( mTwoDLabelSceneEntity ); // see https://bugreports.qt.io/browse/QTBUG-77139
-      connect( mTextX, &Qt3DExtras::QText2DEntity::textChanged, this, &Qgs3DAxis::onTextXChanged );
+      connect( mTextX, &Qt3DExtras::QText2DEntity::textChanged, this, [this]( const QString & text )
+      {
+        updateAxisLabelText( mTextX, text );
+      } );
       mTextTransformX = new Qt3DCore::QTransform();
       mTextCoordX = QVector3D( mCylinderLength + coneLength / 2.0f, 0.0f, 0.0f );
 
@@ -929,7 +939,10 @@ void Qgs3DAxis::createAxis( Qt::Axis axisType )
     case Qt::Axis::YAxis:
       mTextY = new Qt3DExtras::QText2DEntity( );  // object initialization in two step:
       mTextY->setParent( mTwoDLabelSceneEntity ); // see https://bugreports.qt.io/browse/QTBUG-77139
-      connect( mTextY, &Qt3DExtras::QText2DEntity::textChanged, this, &Qgs3DAxis::onTextYChanged );
+      connect( mTextY, &Qt3DExtras::QText2DEntity::textChanged, this, [this]( const QString & text )
+      {
+        updateAxisLabelText( mTextY, text );
+      } );
       mTextTransformY = new Qt3DCore::QTransform();
       mTextCoordY = QVector3D( 0.0f, mCylinderLength + coneLength / 2.0f, 0.0f );
 
@@ -943,7 +956,10 @@ void Qgs3DAxis::createAxis( Qt::Axis axisType )
     case Qt::Axis::ZAxis:
       mTextZ = new Qt3DExtras::QText2DEntity( );  // object initialization in two step:
       mTextZ->setParent( mTwoDLabelSceneEntity ); // see https://bugreports.qt.io/browse/QTBUG-77139
-      connect( mTextZ, &Qt3DExtras::QText2DEntity::textChanged, this, &Qgs3DAxis::onTextZChanged );
+      connect( mTextZ, &Qt3DExtras::QText2DEntity::textChanged, this, [this]( const QString & text )
+      {
+        updateAxisLabelText( mTextZ, text );
+      } );
       mTextTransformZ = new Qt3DCore::QTransform();
       mTextCoordZ = QVector3D( 0.0f, 0.0f, mCylinderLength + coneLength / 2.0f );
 
@@ -1071,7 +1087,7 @@ void Qgs3DAxis::onAxisViewportSizeUpdate( int )
   {
     QgsDebugMsgLevel( "viewport takes too much place into the 3d view, disabling it", 2 );
     // take too much place into the 3d view
-    mAxisViewport->setEnabled( false );
+    mViewport->setEnabled( false );
     setEnableCube( false );
     setEnableAxis( false );
   }
@@ -1081,39 +1097,41 @@ void Qgs3DAxis::onAxisViewportSizeUpdate( int )
     mAxisScaleFactor = viewportPixelSize / defaultViewportPixelSize;
     QgsDebugMsgLevel( QString( "3DAxis viewport mAxisScaleFactor %1" ).arg( mAxisScaleFactor ), 2 );
 
-    if ( ! mAxisViewport->isEnabled() )
+    if ( ! mViewport->isEnabled() )
     {
       if ( settings.mode() == Qgs3DAxisSettings::Mode::Crs )
         setEnableAxis( true );
       else if ( settings.mode() == Qgs3DAxisSettings::Mode::Cube )
         setEnableCube( true );
     }
-    mAxisViewport->setEnabled( true );
+    mViewport->setEnabled( true );
 
     float xRatio = 1.0f;
     float yRatio = 1.0f;
     if ( settings.horizontalPosition() == Qt::AnchorPoint::AnchorLeft )
       xRatio = 0.0f;
     else if ( settings.horizontalPosition() == Qt::AnchorPoint::AnchorHorizontalCenter )
-      xRatio = 0.5f - widthRatio / 2.0f;
+      xRatio = 0.5f - static_cast<float>( widthRatio ) / 2.0f;
     else
-      xRatio = 1.0f - widthRatio;
+      xRatio = 1.0f - static_cast<float>( widthRatio );
 
     if ( settings.verticalPosition() == Qt::AnchorPoint::AnchorTop )
       yRatio = 0.0f;
     else if ( settings.verticalPosition() == Qt::AnchorPoint::AnchorVerticalCenter )
-      yRatio = 0.5f - heightRatio / 2.0f;
+      yRatio = 0.5f - static_cast<float>( heightRatio ) / 2.0f;
     else
-      yRatio = 1.0f - heightRatio;
+      yRatio = 1.0f - static_cast<float>( heightRatio );
 
     QgsDebugMsgLevel( QString( "Qgs3DAxis: update viewport: %1 x %2 x %3 x %4" ).arg( xRatio ).arg( yRatio ).arg( widthRatio ).arg( heightRatio ), 2 );
-    mAxisViewport->setNormalizedRect( QRectF( xRatio, yRatio, widthRatio, heightRatio ) );
+    mViewport->setNormalizedRect( QRectF( xRatio, yRatio, widthRatio, heightRatio ) );
 
     if ( settings.mode() == Qgs3DAxisSettings::Mode::Crs )
     {
+      const float halfWidthSize = static_cast<float>( windowWidth * widthRatio / 2.0 );
+      const float halfHeightSize = static_cast<float>( windowWidth * widthRatio / 2.0 );
       mTwoDLabelCamera->lens()->setOrthographicProjection(
-        -windowWidth / 2.0f, windowWidth / 2.0f,
-        -windowHeight / 2.0f, windowHeight / 2.0f,
+        -halfWidthSize, halfWidthSize,
+        -halfHeightSize, halfHeightSize,
         mTwoDLabelCamera->lens()->nearPlane(), mTwoDLabelCamera->lens()->farPlane() );
 
       updateAxisLabelPosition();
@@ -1143,7 +1161,7 @@ void Qgs3DAxis::onCameraUpdate( )
     }
     else
     {
-      mAxisCamera->setPosition( mainCameraShift * mCylinderLength * 10.0 );
+      mAxisCamera->setPosition( mainCameraShift * mCylinderLength * 9.0 );
     }
 
     if ( mAxisRoot->isEnabled() )
@@ -1157,49 +1175,31 @@ void Qgs3DAxis::updateAxisLabelPosition()
 {
   if ( mTextTransformX && mTextTransformY && mTextTransformZ )
   {
-    mTextTransformX->setTranslation( from3DTo2DLabelPosition( mTextCoordX * mAxisScaleFactor, mAxisCamera,
-                                     mAxisViewport, mTwoDLabelCamera, mTwoDLabelViewport,
-                                     mCanvas->size() ) );
-    onTextXChanged( mTextX->text() );
+    mTextTransformX->setTranslation( from3DTo2DLabelPosition( mTextCoordX * static_cast<float>( mAxisScaleFactor ), mAxisCamera, mTwoDLabelCamera ) );
+    updateAxisLabelText( mTextX, mTextX->text() );
 
-    mTextTransformY->setTranslation( from3DTo2DLabelPosition( mTextCoordY * mAxisScaleFactor, mAxisCamera,
-                                     mAxisViewport, mTwoDLabelCamera, mTwoDLabelViewport,
-                                     mCanvas->size() ) );
-    onTextYChanged( mTextY->text() );
+    mTextTransformY->setTranslation( from3DTo2DLabelPosition( mTextCoordY * static_cast<float>( mAxisScaleFactor ), mAxisCamera, mTwoDLabelCamera ) );
+    updateAxisLabelText( mTextY, mTextY->text() );
 
-    mTextTransformZ->setTranslation( from3DTo2DLabelPosition( mTextCoordZ * mAxisScaleFactor, mAxisCamera,
-                                     mAxisViewport, mTwoDLabelCamera, mTwoDLabelViewport,
-                                     mCanvas->size() ) );
-    onTextZChanged( mTextZ->text() );
+    mTextTransformZ->setTranslation( from3DTo2DLabelPosition( mTextCoordZ * static_cast<float>( mAxisScaleFactor ), mAxisCamera, mTwoDLabelCamera ) );
+    updateAxisLabelText( mTextZ, mTextZ->text() );
   }
 }
 
-void Qgs3DAxis::onTextXChanged( const QString &text )
+void Qgs3DAxis::updateAxisLabelText( Qt3DExtras::QText2DEntity *textEntity, const QString &text )
 {
-  QFont f = QFont( "monospace", mAxisScaleFactor *  mFontSize ); // TODO: should use outlined font
-  f.setWeight( QFont::Weight::Black );
-  f.setStyleStrategy( QFont::StyleStrategy::ForceOutline );
-  mTextX->setFont( f );
-  mTextX->setWidth( mAxisScaleFactor * mFontSize * text.length() );
-  mTextX->setHeight( mAxisScaleFactor * mFontSize * 1.5f );
+  const float scaledFontSize = static_cast<float>( mAxisScaleFactor ) * static_cast<float>( mFontSize );
+  const QFont font = createFont( static_cast<int>( std::round( scaledFontSize ) ) );
+  textEntity->setFont( font );
+  textEntity->setWidth( scaledFontSize * static_cast<float>( text.length() ) );
+  textEntity->setHeight( 1.5f * scaledFontSize );
 }
 
-void Qgs3DAxis::onTextYChanged( const QString &text )
+QFont Qgs3DAxis::createFont( int pointSize )
 {
-  QFont f = QFont( "monospace", mAxisScaleFactor *  mFontSize ); // TODO: should use outlined font
-  f.setWeight( QFont::Weight::Black );
-  f.setStyleStrategy( QFont::StyleStrategy::ForceOutline );
-  mTextY->setFont( f );
-  mTextY->setWidth( mAxisScaleFactor * mFontSize * text.length() );
-  mTextY->setHeight( mAxisScaleFactor * mFontSize * 1.5f );
-}
-
-void Qgs3DAxis::onTextZChanged( const QString &text )
-{
-  QFont f = QFont( "monospace", mAxisScaleFactor *  mFontSize ); // TODO: should use outlined font
-  f.setWeight( QFont::Weight::Black );
-  f.setStyleStrategy( QFont::StyleStrategy::ForceOutline );
-  mTextZ->setFont( f );
-  mTextZ->setWidth( mAxisScaleFactor * mFontSize * text.length() );
-  mTextZ->setHeight( mAxisScaleFactor * mFontSize * 1.5f );
+  QFont font = QFontDatabase::systemFont( QFontDatabase::FixedFont );
+  font.setPointSize( pointSize );
+  font.setWeight( QFont::Weight::Black );
+  font.setStyleStrategy( QFont::StyleStrategy::ForceOutline );
+  return font;
 }
