@@ -48,9 +48,10 @@
 #include "qgs3dutils.h"
 #include "qgsabstract3drenderer.h"
 #include "qgscameracontroller.h"
-#include "qgschunkedentity_p.h"
-#include "qgschunknode_p.h"
+#include "qgschunkedentity.h"
+#include "qgschunknode.h"
 #include "qgseventtracing.h"
+#include "qgsmaterial.h"
 #include "qgsmeshlayer.h"
 #include "qgsmeshlayer3drenderer.h"
 #include "qgspoint3dsymbol.h"
@@ -58,7 +59,7 @@
 #include "qgspointcloudlayer.h"
 #include "qgspointcloudlayer3drenderer.h"
 #include "qgssourcecache.h"
-#include "qgsterrainentity_p.h"
+#include "qgsterrainentity.h"
 #include "qgsterraingenerator.h"
 #include "qgstiledscenelayer.h"
 #include "qgstiledscenelayer3drenderer.h"
@@ -322,6 +323,12 @@ void Qgs3DMapScene::onCameraChanged()
 
 void Qgs3DMapScene::updateScene( bool forceUpdate )
 {
+  if ( !mSceneUpdatesEnabled )
+  {
+    QgsDebugMsgLevel( "Scene update skipped", 2 );
+    return;
+  }
+
   if ( forceUpdate )
     QgsEventTracing::addEvent( QgsEventTracing::Instant, QStringLiteral( "3D" ), QStringLiteral( "Update Scene" ) );
 
@@ -469,7 +476,7 @@ void Qgs3DMapScene::createTerrainDeferred()
     const QgsAABB clippingBbox = Qgs3DUtils::mapToWorldExtent( mMap.extent(), rootBbox.zMin, rootBbox.zMax, mMap.origin() );
     mMap.terrainGenerator()->setupQuadtree( rootBbox, rootError, maxZoomLevel, clippingBbox );
 
-    mTerrain = new QgsTerrainEntity( mMap );
+    mTerrain = new QgsTerrainEntity( &mMap );
     mTerrain->setParent( this );
     mTerrain->setShowBoundingBoxes( mMap.showTerrainBoundingBoxes() );
 
@@ -477,6 +484,11 @@ void Qgs3DMapScene::createTerrainDeferred()
 
     connect( mTerrain, &QgsChunkedEntity::pendingJobsCountChanged, this, &Qgs3DMapScene::totalPendingJobsCountChanged );
     connect( mTerrain, &QgsTerrainEntity::pendingJobsCountChanged, this, &Qgs3DMapScene::terrainPendingJobsCountChanged );
+    connect( mTerrain, &Qgs3DMapSceneEntity::newEntityCreated, this, [this]( Qt3DCore::QEntity * entity )
+    {
+      // enable clipping on the terrain if necessary
+      handleClippingOnEntity( entity );
+    } );
   }
   else
   {
@@ -645,7 +657,7 @@ void Qgs3DMapScene::addLayerEntity( QgsMapLayer *layer )
       tiledSceneRenderer->setLayer( static_cast<QgsTiledSceneLayer *>( layer ) );
     }
 
-    Qt3DCore::QEntity *newEntity = renderer->createEntity( mMap );
+    Qt3DCore::QEntity *newEntity = renderer->createEntity( &mMap );
     if ( newEntity )
     {
       newEntity->setParent( this );
@@ -732,6 +744,9 @@ void Qgs3DMapScene::removeLayerEntity( QgsMapLayer *layer )
 
 void Qgs3DMapScene::finalizeNewEntity( Qt3DCore::QEntity *newEntity )
 {
+  // set clip planes on the new entity if necessary
+  handleClippingOnEntity( newEntity );
+
   // this is probably not the best place for material-specific configuration,
   // maybe this could be more generalized when other materials need some specific treatment
   const QList< QgsLineMaterial *> childLineMaterials = newEntity->findChildren<QgsLineMaterial *>();
@@ -880,7 +895,7 @@ void Qgs3DMapScene::updateSceneState()
 void Qgs3DMapScene::onSkyboxSettingsChanged()
 {
   QgsSkyboxSettings skyboxSettings = mMap.skyboxSettings();
-  if ( mSkybox != nullptr )
+  if ( mSkybox )
   {
     mSkybox->deleteLater();
     mSkybox = nullptr;
@@ -1161,3 +1176,74 @@ void Qgs3DMapScene::on3DAxisSettingsChanged()
     }
   }
 }
+
+void Qgs3DMapScene::handleClippingOnEntity( QEntity *entity ) const
+{
+  if ( mClipPlanesEquations.isEmpty() ) // no clip plane equations, disable clipping
+  {
+    for ( QgsMaterial *material : entity->componentsOfType<QgsMaterial>() )
+    {
+      material->disableClipping();
+    }
+  }
+  else // enable clipping
+  {
+    for ( QgsMaterial *material : entity->componentsOfType<QgsMaterial>() )
+    {
+      material->enableClipping( mClipPlanesEquations );
+    }
+  }
+
+  // recursive call
+  // enable or disable clipping on the children accordingly
+  for ( QObject *child : entity->children() )
+  {
+    Qt3DCore::QEntity *childEntity = qobject_cast<Qt3DCore::QEntity *>( child );
+    if ( childEntity )
+    {
+      handleClippingOnEntity( childEntity );
+    }
+  }
+}
+
+void Qgs3DMapScene::handleClippingOnAllEntities() const
+{
+  // Need to loop mLayerEntities instead of mSceneEntities to handle entities
+  // which do no inherit from Qgs3DMapSceneEntity. For example, mesh entities.
+  for ( auto it = mLayerEntities.constBegin(); it != mLayerEntities.constEnd(); ++it )
+  {
+    handleClippingOnEntity( it.value() );
+  }
+  if ( mTerrain )
+  {
+    handleClippingOnEntity( mTerrain );
+  }
+}
+
+void Qgs3DMapScene::enableClipping( const QList<QVector4D> &clipPlaneEquations )
+{
+  if ( clipPlaneEquations.size() > 8 )
+  {
+    QgsDebugMsgLevel( QStringLiteral( "Qgs3DMapScene::enableClipping: it is not possible to use more than 8 clipping planes." ), 2 );
+  }
+  mClipPlanesEquations = clipPlaneEquations.mid( 0, 8 );
+
+  // enable the clip planes on the framegraph
+  QgsFrameGraph *frameGraph = mEngine->frameGraph();
+  frameGraph->addClipPlanes( clipPlaneEquations.size() );
+
+  // Enable the clip planes for the material of each entity.
+  handleClippingOnAllEntities();
+}
+
+void Qgs3DMapScene::disableClipping()
+{
+  mClipPlanesEquations.clear();
+
+  // disable the clip planes on the framegraph
+  QgsFrameGraph *frameGraph = mEngine->frameGraph();
+  frameGraph->removeClipPlanes();
+
+  // Disable the clip planes for the material of each entity.
+  handleClippingOnAllEntities();
+}
