@@ -43,25 +43,17 @@ typedef Qt3DCore::QGeometry Qt3DQGeometry;
 #include <Qt3DRender/QTechnique>
 #include <Qt3DRender/QGraphicsApiFilter>
 #include <Qt3DRender/QBlendEquation>
+#include <Qt3DRender/QColorMask>
 #include <Qt3DRender/QSortPolicy>
+#include <Qt3DRender/QPointSize>
+#include <Qt3DRender/QSeamlessCubemap>
 #include <Qt3DRender/QNoDepthMask>
 #include <Qt3DRender/QBlendEquationArguments>
-
-Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructTexturesPreviewPass()
-{
-  mPreviewLayerFilter = new Qt3DRender::QLayerFilter;
-  mPreviewLayerFilter->addLayer( mPreviewLayer );
-
-  mPreviewRenderStateSet = new Qt3DRender::QRenderStateSet( mPreviewLayerFilter );
-  mPreviewDepthTest = new Qt3DRender::QDepthTest;
-  mPreviewDepthTest->setDepthFunction( Qt3DRender::QDepthTest::Always );
-  mPreviewRenderStateSet->addRenderState( mPreviewDepthTest );
-  mPreviewCullFace = new Qt3DRender::QCullFace;
-  mPreviewCullFace->setMode( Qt3DRender::QCullFace::NoCulling );
-  mPreviewRenderStateSet->addRenderState( mPreviewCullFace );
-
-  return mPreviewLayerFilter;
-}
+#include <Qt3DExtras/QTextureMaterial>
+#include <Qt3DRender/QAbstractTexture>
+#include "qgsfgutils.h"
+#include <Qt3DRender/QNoDraw>
+#include <Qt3DRender/QClipPlane>
 
 Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructForwardRenderPass()
 {
@@ -77,6 +69,10 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructForwardRenderPass()
   //                         |  QLayerFilter   |  (using mForwardRenderLayer)
   //                         +-----------------+
   //                                  |
+  //                         +-----------------+
+  //                         | QRenderStateSet |  define clip planes
+  //                         +-----------------+
+  //                                  |
   //                      +-----------------------+
   //                      | QRenderTargetSelector | (write mForwardColorTexture + mForwardDepthTexture)
   //                      +-----------------------+
@@ -91,20 +87,25 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructForwardRenderPass()
   //  | QRenderStateSet |  cull back faces         |  QSortPolicy    |  back to front
   //  +-----------------+                          +-----------------+
   //         |                                              |
-  //  +-----------------+                          +-----------------+  use depth test
-  //  | QFrustumCulling |                          | QRenderStateSet |  don't write depths
-  //  +-----------------+                          +-----------------+  no culling
-  //         |                                                          use alpha blending
-  //  +-----------------+
-  //  |  QClearBuffers  |  color and depth
+  //  +-----------------+              +--------------------+--------------------+
+  //  | QFrustumCulling |              |                                         |
+  //  +-----------------+     +-----------------+  use depth tests      +-----------------+  use depth tests
+  //         |                | QRenderStateSet |  don't write depths   | QRenderStateSet |  write depths
+  //         |                +-----------------+  write colors         +-----------------+  don't write colors
+  //  +-----------------+                          use alpha blending                        don't use alpha blending
+  //  |  QClearBuffers  |  color and depth         no culling                                no culling
   //  +-----------------+
 
   mMainCameraSelector = new Qt3DRender::QCameraSelector;
+  mMainCameraSelector->setObjectName( "Forward render pass CameraSelector" );
   mMainCameraSelector->setCamera( mMainCamera );
 
   mForwardRenderLayerFilter = new Qt3DRender::QLayerFilter( mMainCameraSelector );
   mForwardRenderLayerFilter->addLayer( mForwardRenderLayer );
 
+  mClipRenderStateSet = new Qt3DRender::QRenderStateSet( mForwardRenderLayerFilter );
+  mClipRenderStateSet->setObjectName( "Forward render pass Clip Plane RenderStateSet" );
+
   mForwardColorTexture = new Qt3DRender::QTexture2D;
   mForwardColorTexture->setWidth( mSize.width() );
   mForwardColorTexture->setHeight( mSize.height() );
@@ -135,30 +136,32 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructForwardRenderPass()
   forwardRenderTargetColorOutput->setTexture( mForwardColorTexture );
   forwardRenderTarget->addOutput( forwardRenderTargetColorOutput );
 
-  mForwardRenderTargetSelector = new Qt3DRender::QRenderTargetSelector( mForwardRenderLayerFilter );
+  mForwardRenderTargetSelector = new Qt3DRender::QRenderTargetSelector( mClipRenderStateSet );
   mForwardRenderTargetSelector->setTarget( forwardRenderTarget );
 
+  // first branch: opaque layer filter
   Qt3DRender::QLayerFilter *opaqueObjectsFilter = new Qt3DRender::QLayerFilter( mForwardRenderTargetSelector );
   opaqueObjectsFilter->addLayer( mTransparentObjectsPassLayer );
   opaqueObjectsFilter->setFilterMode( Qt3DRender::QLayerFilter::DiscardAnyMatchingLayers );
 
-  Qt3DRender::QRenderStateSet *forwaredRenderStateSet = new Qt3DRender::QRenderStateSet( opaqueObjectsFilter );
+  Qt3DRender::QRenderStateSet *forwardedRenderStateSet = new Qt3DRender::QRenderStateSet( opaqueObjectsFilter );
 
   Qt3DRender::QDepthTest *depthTest = new Qt3DRender::QDepthTest;
   depthTest->setDepthFunction( Qt3DRender::QDepthTest::Less );
-  forwaredRenderStateSet->addRenderState( depthTest );
+  forwardedRenderStateSet->addRenderState( depthTest );
 
   Qt3DRender::QCullFace *cullFace = new Qt3DRender::QCullFace;
   cullFace->setMode( Qt3DRender::QCullFace::CullingMode::Back );
-  forwaredRenderStateSet->addRenderState( cullFace );
+  forwardedRenderStateSet->addRenderState( cullFace );
 
-  mFrustumCulling = new Qt3DRender::QFrustumCulling( forwaredRenderStateSet );
+  mFrustumCulling = new Qt3DRender::QFrustumCulling( forwardedRenderStateSet );
 
   mForwardClearBuffers = new Qt3DRender::QClearBuffers( mFrustumCulling );
   mForwardClearBuffers->setClearColor( QColor::fromRgbF( 0.0, 0.0, 1.0, 1.0 ) );
   mForwardClearBuffers->setBuffers( Qt3DRender::QClearBuffers::ColorDepthBuffer );
   mForwardClearBuffers->setClearDepthValue( 1.0f );
 
+  // second branch: transparent layer filter - color
   Qt3DRender::QLayerFilter *transparentObjectsLayerFilter = new Qt3DRender::QLayerFilter( mForwardRenderTargetSelector );
   transparentObjectsLayerFilter->addLayer( mTransparentObjectsPassLayer );
   transparentObjectsLayerFilter->setFilterMode( Qt3DRender::QLayerFilter::AcceptAnyMatchingLayers );
@@ -168,35 +171,52 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructForwardRenderPass()
   sortTypes.push_back( Qt3DRender::QSortPolicy::BackToFront );
   sortPolicy->setSortTypes( sortTypes );
 
-  Qt3DRender::QRenderStateSet *transparentObjectsRenderStateSet = new Qt3DRender::QRenderStateSet( sortPolicy );
+  Qt3DRender::QRenderStateSet *transparentObjectsRenderStateSetColor = new Qt3DRender::QRenderStateSet( sortPolicy );
   {
     Qt3DRender::QDepthTest *depthTest = new Qt3DRender::QDepthTest;
     depthTest->setDepthFunction( Qt3DRender::QDepthTest::Less );
-    transparentObjectsRenderStateSet->addRenderState( depthTest );
+    transparentObjectsRenderStateSetColor->addRenderState( depthTest );
 
     Qt3DRender::QNoDepthMask *noDepthMask = new Qt3DRender::QNoDepthMask;
-    transparentObjectsRenderStateSet->addRenderState( noDepthMask );
+    transparentObjectsRenderStateSetColor->addRenderState( noDepthMask );
 
     Qt3DRender::QCullFace *cullFace = new Qt3DRender::QCullFace;
     cullFace->setMode( Qt3DRender::QCullFace::CullingMode::NoCulling );
-    transparentObjectsRenderStateSet->addRenderState( cullFace );
+    transparentObjectsRenderStateSetColor->addRenderState( cullFace );
 
     Qt3DRender::QBlendEquation *blendEquation = new Qt3DRender::QBlendEquation;
     blendEquation->setBlendFunction( Qt3DRender::QBlendEquation::Add );
-    transparentObjectsRenderStateSet->addRenderState( blendEquation );
+    transparentObjectsRenderStateSetColor->addRenderState( blendEquation );
 
-    Qt3DRender::QBlendEquationArguments *blenEquationArgs = new Qt3DRender::QBlendEquationArguments;
-    blenEquationArgs->setSourceRgb( Qt3DRender::QBlendEquationArguments::Blending::One );
-    blenEquationArgs->setDestinationRgb( Qt3DRender::QBlendEquationArguments::Blending::OneMinusSource1Alpha );
-    blenEquationArgs->setSourceAlpha( Qt3DRender::QBlendEquationArguments::Blending::One );
-    blenEquationArgs->setDestinationAlpha( Qt3DRender::QBlendEquationArguments::Blending::OneMinusSource1Alpha );
-    transparentObjectsRenderStateSet->addRenderState( blenEquationArgs );
+    Qt3DRender::QBlendEquationArguments *blendEquationArgs = new Qt3DRender::QBlendEquationArguments;
+    blendEquationArgs->setSourceRgb( Qt3DRender::QBlendEquationArguments::Blending::SourceAlpha );
+    blendEquationArgs->setDestinationRgb( Qt3DRender::QBlendEquationArguments::Blending::OneMinusSourceAlpha );
+    transparentObjectsRenderStateSetColor->addRenderState( blendEquationArgs );
+  }
+
+  // third branch: transparent layer filter - depth
+  Qt3DRender::QRenderStateSet *transparentObjectsRenderStateSetDepth = new Qt3DRender::QRenderStateSet( sortPolicy );
+  {
+    Qt3DRender::QDepthTest *depthTest = new Qt3DRender::QDepthTest;
+    depthTest->setDepthFunction( Qt3DRender::QDepthTest::Less );
+    transparentObjectsRenderStateSetDepth->addRenderState( depthTest );
+
+    Qt3DRender::QColorMask *noColorMask = new Qt3DRender::QColorMask;
+    noColorMask->setAlphaMasked( false );
+    noColorMask->setRedMasked( false );
+    noColorMask->setGreenMasked( false );
+    noColorMask->setBlueMasked( false );
+    transparentObjectsRenderStateSetDepth->addRenderState( noColorMask );
+
+    Qt3DRender::QCullFace *cullFace = new Qt3DRender::QCullFace;
+    cullFace->setMode( Qt3DRender::QCullFace::CullingMode::NoCulling );
+    transparentObjectsRenderStateSetDepth->addRenderState( cullFace );
   }
 
   mDebugOverlay = new Qt3DRender::QDebugOverlay( mForwardClearBuffers );
   mDebugOverlay->setEnabled( false );
 
-  // cppcheck wrongly believes transparentObjectsRenderStateSet will leak
+  // cppcheck wrongly believes transparentObjectsRenderStateSetColor and transparentObjectsRenderStateSetDepth will leak
   // cppcheck-suppress memleak
   return mMainCameraSelector;
 }
@@ -204,6 +224,7 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructForwardRenderPass()
 Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructShadowRenderPass()
 {
   mLightCameraSelectorShadowPass = new Qt3DRender::QCameraSelector;
+  mLightCameraSelectorShadowPass->setObjectName( "Shadow render pass CameraSelector" );
   mLightCameraSelectorShadowPass->setCamera( mLightCamera );
 
   mShadowSceneEntitiesFilter = new Qt3DRender::QLayerFilter( mLightCameraSelectorShadowPass );
@@ -250,16 +271,57 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructShadowRenderPass()
   return mLightCameraSelectorShadowPass;
 }
 
+Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructSubPostPassForTexturesPreview()
+{
+  Qt3DRender::QLayerFilter *layerFilter = new Qt3DRender::QLayerFilter;
+  layerFilter->setObjectName( "Sub pass TexturesPreview" );
+  layerFilter->addLayer( mPreviewLayer );
+
+  Qt3DRender::QRenderStateSet *renderStateSet = new Qt3DRender::QRenderStateSet( layerFilter );
+  Qt3DRender::QDepthTest *depthTest = new Qt3DRender::QDepthTest;
+  depthTest->setDepthFunction( Qt3DRender::QDepthTest::Always );
+  renderStateSet->addRenderState( depthTest );
+  Qt3DRender::QCullFace *cullFace = new Qt3DRender::QCullFace;
+  cullFace->setMode( Qt3DRender::QCullFace::NoCulling );
+  renderStateSet->addRenderState( cullFace );
+
+  return layerFilter;
+}
+
+Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructSubPostPassForProcessing()
+{
+  Qt3DRender::QCameraSelector *cameraSelector = new Qt3DRender::QCameraSelector;
+  cameraSelector->setObjectName( "Sub pass Postprocessing" );
+  cameraSelector->setCamera( mLightCamera );
+
+  Qt3DRender::QLayerFilter *layerFilter = new Qt3DRender::QLayerFilter( cameraSelector );
+
+  // could be the first of this branch
+  new Qt3DRender::QClearBuffers( layerFilter );
+
+  Qt3DRender::QLayer *postProcessingLayer = new Qt3DRender::QLayer();
+  mPostprocessingEntity = new QgsPostprocessingEntity( this, postProcessingLayer, mRootEntity );
+  layerFilter->addLayer( postProcessingLayer );
+  mPostprocessingEntity->setObjectName( "PostProcessingPassEntity" );
+
+  return cameraSelector;
+}
+
+Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructSubPostPassForRenderCapture()
+{
+  Qt3DRender::QFrameGraphNode *top = new Qt3DRender::QNoDraw;
+  top->setObjectName( "Sub pass RenderCapture" );
+
+  mRenderCapture = new Qt3DRender::QRenderCapture( top );
+
+  return top;
+}
+
 Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructPostprocessingPass()
 {
-  mPostProcessingCameraSelector = new Qt3DRender::QCameraSelector;
-  mPostProcessingCameraSelector->setCamera( mLightCamera );
-
-  mPostprocessPassLayerFilter = new Qt3DRender::QLayerFilter( mPostProcessingCameraSelector );
-
-  mPostprocessClearBuffers = new Qt3DRender::QClearBuffers( mPostprocessPassLayerFilter );
-
-  mRenderCaptureTargetSelector = new Qt3DRender::QRenderTargetSelector( mPostprocessClearBuffers );
+  mRenderCaptureTargetSelector = new Qt3DRender::QRenderTargetSelector;
+  mRenderCaptureTargetSelector->setObjectName( "Postprocessing render pass" );
+  mRenderCaptureTargetSelector->setEnabled( mRenderCaptureEnabled );
 
   Qt3DRender::QRenderTarget *renderTarget = new Qt3DRender::QRenderTarget( mRenderCaptureTargetSelector );
 
@@ -276,6 +338,7 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructPostprocessingPass()
   mRenderCaptureColorTexture->setFormat( Qt3DRender::QAbstractTexture::RGB8_UNorm );
   mRenderCaptureColorTexture->setMinificationFilter( Qt3DRender::QAbstractTexture::Linear );
   mRenderCaptureColorTexture->setMagnificationFilter( Qt3DRender::QAbstractTexture::Linear );
+  mRenderCaptureColorTexture->setObjectName( "PostProcessingPass::ColorTarget" );
 
   // Hook the texture up to our output, and the output up to this object.
   colorOutput->setTexture( mRenderCaptureColorTexture );
@@ -291,23 +354,25 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructPostprocessingPass()
   mRenderCaptureDepthTexture->setMagnificationFilter( Qt3DRender::QAbstractTexture::Linear );
   mRenderCaptureDepthTexture->setComparisonFunction( Qt3DRender::QAbstractTexture::CompareLessEqual );
   mRenderCaptureDepthTexture->setComparisonMode( Qt3DRender::QAbstractTexture::CompareRefToTexture );
+  mRenderCaptureDepthTexture->setObjectName( "PostProcessingPass::DepthTarget" );
 
   depthOutput->setTexture( mRenderCaptureDepthTexture );
   renderTarget->addOutput( depthOutput );
 
   mRenderCaptureTargetSelector->setTarget( renderTarget );
 
-  mRenderCapture = new Qt3DRender::QRenderCapture( mRenderCaptureTargetSelector );
+  // sub passes:
+  constructSubPostPassForProcessing()->setParent( mRenderCaptureTargetSelector );
+  constructSubPostPassForTexturesPreview()->setParent( mRenderCaptureTargetSelector );
+  constructSubPostPassForRenderCapture()->setParent( mRenderCaptureTargetSelector );
 
-  mPostprocessingEntity = new QgsPostprocessingEntity( this, mRootEntity );
-  mPostprocessPassLayerFilter->addLayer( mPostprocessingEntity->layer() );
-
-  return mPostProcessingCameraSelector;
+  return mRenderCaptureTargetSelector;
 }
 
 Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructAmbientOcclusionRenderPass()
 {
   mAmbientOcclusionRenderCameraSelector = new Qt3DRender::QCameraSelector;
+  mAmbientOcclusionRenderCameraSelector->setObjectName( "AmbientOcclusion render pass CameraSelector" );
   mAmbientOcclusionRenderCameraSelector->setCamera( mMainCamera );
 
   mAmbientOcclusionRenderStateSet = new Qt3DRender::QRenderStateSet( mAmbientOcclusionRenderCameraSelector );
@@ -323,13 +388,13 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructAmbientOcclusionRenderPass(
   mAmbientOcclusionRenderLayerFilter = new Qt3DRender::QLayerFilter( mAmbientOcclusionRenderStateSet );
 
   mAmbientOcclusionRenderCaptureTargetSelector = new Qt3DRender::QRenderTargetSelector( mAmbientOcclusionRenderLayerFilter );
-  Qt3DRender::QRenderTarget *depthRenderTarget = new Qt3DRender::QRenderTarget( mAmbientOcclusionRenderCaptureTargetSelector );
+  Qt3DRender::QRenderTarget *colorRenderTarget = new Qt3DRender::QRenderTarget( mAmbientOcclusionRenderCaptureTargetSelector );
 
   // The lifetime of the objects created here is managed
   // automatically, as they become children of this object.
 
   // Create a render target output for rendering color.
-  Qt3DRender::QRenderTargetOutput *colorOutput = new Qt3DRender::QRenderTargetOutput( depthRenderTarget );
+  Qt3DRender::QRenderTargetOutput *colorOutput = new Qt3DRender::QRenderTargetOutput( colorRenderTarget );
   colorOutput->setAttachmentPoint( Qt3DRender::QRenderTargetOutput::Color0 );
 
   // Create a texture to render into.
@@ -341,12 +406,13 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructAmbientOcclusionRenderPass(
 
   // Hook the texture up to our output, and the output up to this object.
   colorOutput->setTexture( mAmbientOcclusionRenderTexture );
-  depthRenderTarget->addOutput( colorOutput );
+  colorRenderTarget->addOutput( colorOutput );
 
-  mAmbientOcclusionRenderCaptureTargetSelector->setTarget( depthRenderTarget );
+  mAmbientOcclusionRenderCaptureTargetSelector->setTarget( colorRenderTarget );
 
-  mAmbientOcclusionRenderEntity = new QgsAmbientOcclusionRenderEntity( mForwardDepthTexture, mMainCamera, mRootEntity );
-  mAmbientOcclusionRenderLayerFilter->addLayer( mAmbientOcclusionRenderEntity->layer() );
+  Qt3DRender::QLayer *ambientOcclusionRenderLayer = new Qt3DRender::QLayer();
+  mAmbientOcclusionRenderEntity = new QgsAmbientOcclusionRenderEntity( mForwardDepthTexture, ambientOcclusionRenderLayer, mMainCamera, mRootEntity );
+  mAmbientOcclusionRenderLayerFilter->addLayer( ambientOcclusionRenderLayer );
 
   return mAmbientOcclusionRenderCameraSelector;
 }
@@ -354,6 +420,7 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructAmbientOcclusionRenderPass(
 Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructAmbientOcclusionBlurPass()
 {
   mAmbientOcclusionBlurCameraSelector = new Qt3DRender::QCameraSelector;
+  mAmbientOcclusionBlurCameraSelector->setObjectName( "AmbientOcclusion blur pass CameraSelector" );
   mAmbientOcclusionBlurCameraSelector->setCamera( mMainCamera );
 
   mAmbientOcclusionBlurStateSet = new Qt3DRender::QRenderStateSet( mAmbientOcclusionBlurCameraSelector );
@@ -391,8 +458,9 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructAmbientOcclusionBlurPass()
 
   mAmbientOcclusionBlurRenderCaptureTargetSelector->setTarget( depthRenderTarget );
 
-  mAmbientOcclusionBlurEntity = new QgsAmbientOcclusionBlurEntity( mAmbientOcclusionRenderTexture, mRootEntity );
-  mAmbientOcclusionBlurLayerFilter->addLayer( mAmbientOcclusionBlurEntity->layer() );
+  Qt3DRender::QLayer *ambientOcclusionBlurLayer = new Qt3DRender::QLayer();
+  mAmbientOcclusionBlurEntity = new QgsAmbientOcclusionBlurEntity( mAmbientOcclusionRenderTexture, ambientOcclusionBlurLayer, mRootEntity );
+  mAmbientOcclusionBlurLayerFilter->addLayer( ambientOcclusionBlurLayer );
 
   return mAmbientOcclusionBlurCameraSelector;
 }
@@ -401,6 +469,7 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructAmbientOcclusionBlurPass()
 Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructRubberBandsPass()
 {
   mRubberBandsCameraSelector = new Qt3DRender::QCameraSelector;
+  mRubberBandsCameraSelector->setObjectName( "RubberBands Pass CameraSelector" );
   mRubberBandsCameraSelector->setCamera( mMainCamera );
 
   mRubberBandsLayerFilter = new Qt3DRender::QLayerFilter( mRubberBandsCameraSelector );
@@ -427,6 +496,7 @@ Qt3DRender::QFrameGraphNode *QgsFrameGraph::constructDepthRenderPass()
   // depth buffer render to copy pass
 
   mDepthRenderCameraSelector = new Qt3DRender::QCameraSelector;
+  mDepthRenderCameraSelector->setObjectName( "Depth render view CameraSelector" );
   mDepthRenderCameraSelector->setCamera( mMainCamera );
 
   mDepthRenderStateSet = new Qt3DRender::QRenderStateSet( mDepthRenderCameraSelector );
@@ -598,6 +668,8 @@ QgsFrameGraph::QgsFrameGraph( QSurface *surface, QSize s, Qt3DRender::QCamera *m
   mTransparentObjectsPassLayer = new Qt3DRender::QLayer;
   mRubberBandsLayer = new Qt3DRender::QLayer;
 
+  mRubberBandsLayer->setObjectName( "mRubberBandsLayer" );
+
   mPreviewLayer->setRecursive( true );
   mCastShadowsLayer->setRecursive( true );
   mForwardRenderLayer->setRecursive( true );
@@ -622,6 +694,7 @@ QgsFrameGraph::QgsFrameGraph( QSurface *surface, QSize s, Qt3DRender::QCamera *m
 
   // rubber bands (they should be always on top)
   Qt3DRender::QFrameGraphNode *rubberBandsPass = constructRubberBandsPass();
+  rubberBandsPass->setObjectName( "rubberBandsPass" );
   rubberBandsPass->setParent( mMainViewPort );
 
   // shadow rendering pass
@@ -642,14 +715,12 @@ QgsFrameGraph::QgsFrameGraph( QSurface *surface, QSize s, Qt3DRender::QCamera *m
   // post process
   Qt3DRender::QFrameGraphNode *postprocessingPass = constructPostprocessingPass();
   postprocessingPass->setParent( mMainViewPort );
+  postprocessingPass->setObjectName( "PostProcessingPass" );
 
   mRubberBandsRootEntity = new Qt3DCore::QEntity( mRootEntity );
+  mRubberBandsRootEntity->setObjectName( "mRubberBandsRootEntity" );
   mRubberBandsRootEntity->addComponent( mRubberBandsLayer );
 
-  // textures preview pass
-  Qt3DRender::QFrameGraphNode *previewPass = constructTexturesPreviewPass();
-  previewPass->setParent( mMainViewPort );
-
   Qt3DRender::QParameter *depthMapIsDepthParam = new Qt3DRender::QParameter( "isDepth", true );
   Qt3DRender::QParameter *shadowMapIsDepthParam = new Qt3DRender::QParameter( "isDepth", true );
 
@@ -759,6 +830,25 @@ void QgsFrameGraph::setupDirectionalLight( const QgsDirectionalLightSettings &li
   mPostprocessingEntity->setupDirectionalLight( lightPosition, lightDirection );
 }
 
+QString QgsFrameGraph::dumpFrameGraph() const
+{
+  QObject *top = mRenderSurfaceSelector;
+  while ( top->parent() && dynamic_cast<Qt3DRender::QFrameGraphNode *>( top->parent() ) )
+    top = top->parent();
+
+  QgsFgUtils::FgDumpContext context;
+  context.lowestId = mMainCamera->id().id();
+  QStringList strList = QgsFgUtils::dumpFrameGraph( dynamic_cast<Qt3DRender::QFrameGraphNode *>( top ), context );
+
+  return qPrintable( strList.join( "\n" ) ) + QString( "\n" );
+}
+
+QString QgsFrameGraph::dumpSceneGraph() const
+{
+  QStringList strList = QgsFgUtils::dumpSceneGraph( mRootEntity, QgsFgUtils::FgDumpContext() );
+  return qPrintable( strList.join( "\n" ) ) + QString( "\n" );
+}
+
 void QgsFrameGraph::setClearColor( const QColor &clearColor )
 {
   mForwardClearBuffers->setClearColor( clearColor );
@@ -907,3 +997,29 @@ void QgsFrameGraph::setDebugOverlayEnabled( bool enabled )
 {
   mDebugOverlay->setEnabled( enabled );
 }
+
+void QgsFrameGraph::removeClipPlanes()
+{
+  for ( Qt3DRender::QRenderState *state : mClipRenderStateSet->renderStates() )
+  {
+    if ( qobject_cast<Qt3DRender::QClipPlane *>( state ) )
+    {
+      mClipRenderStateSet->removeRenderState( state );
+    }
+  }
+}
+
+void QgsFrameGraph::addClipPlanes( int nrClipPlanes )
+{
+  // remove existing QClipPlane
+  removeClipPlanes();
+
+  // create new QClipPlane
+  for ( int i = 0; i < nrClipPlanes; ++i )
+  {
+    Qt3DRender::QClipPlane *clipPlane = new Qt3DRender::QClipPlane;
+    clipPlane->setPlaneIndex( i );
+    mClipRenderStateSet->addRenderState( clipPlane );
+  }
+
+}
