@@ -69,7 +69,8 @@ void QgsRasterLayerUniqueValuesReportAlgorithm::initAlgorithm( const QVariantMap
 
 QString QgsRasterLayerUniqueValuesReportAlgorithm::shortHelpString() const
 {
-  return QObject::tr( "This algorithm returns the count and area of each unique value in a given raster layer." );
+  return QObject::tr( "This algorithm returns the count and area of each unique value in a given raster layer. "
+                      "The area calculation is done in the area unit of the layer's CRS." );
 }
 
 QgsRasterLayerUniqueValuesReportAlgorithm *QgsRasterLayerUniqueValuesReportAlgorithm::createInstance() const
