@@ -564,7 +564,6 @@ void QgsPluginRegistry::restoreSessionPlugins( const QString &pluginDirString )
     QgsDebugMsgLevel( QStringLiteral( "Python plugins will be loaded in the following order: " ) + pluginList.join( "," ), 2 );
 
     QStringList corePlugins = QStringList();
-    corePlugins << QStringLiteral( "GdalTools" );
     corePlugins << QStringLiteral( "db_manager" );
     corePlugins << QStringLiteral( "processing" );
     corePlugins << QStringLiteral( "MetaSearch" );
@@ -725,7 +724,10 @@ bool QgsPluginRegistry::isPythonPluginCompatible( const QString &packageName ) c
   const QString supportsQt6 = mPythonUtils->getPluginMetadata( packageName, QStringLiteral( "supportsQt6" ) ).trimmed();
   if ( supportsQt6.compare( QLatin1String( "YES" ), Qt::CaseInsensitive ) != 0 && supportsQt6.compare( QLatin1String( "TRUE" ), Qt::CaseInsensitive ) != 0 )
   {
-    return false;
+    if ( !getenv( "QGIS_DISABLE_SUPPORTS_QT6_CHECK" ) )
+    {
+      return false;
+    }
   }
 #endif
   const QString minVersion = mPythonUtils->getPluginMetadata( packageName, QStringLiteral( "qgisMinimumVersion" ) );
