@@ -38,12 +38,12 @@
 
 ///@cond PRIVATE
 
-class QgsMapRendererTaskGeoPdfExporter : public QgsAbstractGeoPdfExporter
+class QgsMapRendererTaskGeospatialPdfExporter : public QgsAbstractGeospatialPdfExporter
 {
 
   public:
 
-    QgsMapRendererTaskGeoPdfExporter( const QgsMapSettings &ms )
+    QgsMapRendererTaskGeospatialPdfExporter( const QgsMapSettings &ms )
     {
       // collect details upfront, while we are still in the main thread
       const QList< QgsMapLayer * > layers = ms.layers();
@@ -62,7 +62,7 @@ class QgsMapRendererTaskGeoPdfExporter : public QgsAbstractGeoPdfExporter
 
   private:
 
-    QgsAbstractGeoPdfExporter::VectorComponentDetail componentDetailForLayerId( const QString &layerId ) override
+    QgsAbstractGeospatialPdfExporter::VectorComponentDetail componentDetailForLayerId( const QString &layerId ) override
     {
       return mLayerDetails.value( layerId );
     }
@@ -75,7 +75,7 @@ class QgsMapRendererTaskRenderedFeatureHandler : public QgsRenderedFeatureHandle
 {
   public:
 
-    QgsMapRendererTaskRenderedFeatureHandler( QgsMapRendererTaskGeoPdfExporter *exporter, const QgsMapSettings &settings )
+    QgsMapRendererTaskRenderedFeatureHandler( QgsMapRendererTaskGeospatialPdfExporter *exporter, const QgsMapSettings &settings )
       : mExporter( exporter )
       , mMapSettings( settings )
     {
@@ -97,7 +97,7 @@ class QgsMapRendererTaskRenderedFeatureHandler : public QgsRenderedFeatureHandle
       // always convert to multitype, to make things consistent
       transformed.convertToMultiType();
 
-      mExporter->pushRenderedFeature( layerId, QgsAbstractGeoPdfExporter::RenderedFeature( feature, transformed ) );
+      mExporter->pushRenderedFeature( layerId, QgsAbstractGeospatialPdfExporter::RenderedFeature( feature, transformed ) );
     }
 
     QSet<QString> usedAttributes( QgsVectorLayer *, const QgsRenderContext & ) const override
@@ -107,7 +107,7 @@ class QgsMapRendererTaskRenderedFeatureHandler : public QgsRenderedFeatureHandle
 
   private:
 
-    QgsMapRendererTaskGeoPdfExporter *mExporter = nullptr;
+    QgsMapRendererTaskGeospatialPdfExporter *mExporter = nullptr;
     QgsMapSettings mMapSettings;
     //! Transform from output space (pixels) to PDF space (pixels at 72 dpi)
     QTransform mTransform;
@@ -117,14 +117,14 @@ class QgsMapRendererTaskRenderedFeatureHandler : public QgsRenderedFeatureHandle
 ///@endcond
 
 QgsMapRendererTask::QgsMapRendererTask( const QgsMapSettings &ms, const QString &fileName, const QString &fileFormat, const bool forceRaster, QgsTask::Flags flags,
-                                        const bool geoPDF, const QgsAbstractGeoPdfExporter::ExportDetails &geoPdfExportDetails )
+                                        const bool geoPDF, const QgsAbstractGeospatialPdfExporter::ExportDetails &geospatialPdfExportDetails )
   : QgsTask( fileFormat == QLatin1String( "PDF" ) ? tr( "Saving as PDF" ) : tr( "Saving as image" ), flags )
   , mMapSettings( ms )
   , mFileName( fileName )
   , mFileFormat( fileFormat )
   , mForceRaster( forceRaster )
-  , mGeoPDF( geoPDF && mFileFormat == QLatin1String( "PDF" ) && QgsAbstractGeoPdfExporter::geoPDFCreationAvailable() )
-  , mGeoPdfExportDetails( geoPdfExportDetails )
+  , mGeospatialPDF( geoPDF && mFileFormat == QLatin1String( "PDF" ) && QgsAbstractGeospatialPdfExporter::geospatialPDFCreationAvailable() )
+  , mGeospatialPdfExportDetails( geospatialPdfExportDetails )
 {
   if ( mFileFormat == QLatin1String( "PDF" ) && !qgsDoubleNear( mMapSettings.devicePixelRatio(), 1.0 ) )
   {
@@ -178,21 +178,21 @@ bool QgsMapRendererTask::run()
   if ( mErrored )
     return false;
 
-  if ( mGeoPDF )
+  if ( mGeospatialPDF )
   {
-    QList< QgsAbstractGeoPdfExporter::ComponentLayerDetail > pdfComponents;
+    QList< QgsAbstractGeospatialPdfExporter::ComponentLayerDetail > pdfComponents;
 
     QgsMapRendererStagedRenderJob *job = static_cast< QgsMapRendererStagedRenderJob * >( mJob.get() );
     int outputLayer = 1;
     while ( !job->isFinished() )
     {
-      QgsAbstractGeoPdfExporter::ComponentLayerDetail component;
+      QgsAbstractGeospatialPdfExporter::ComponentLayerDetail component;
 
       component.name = QStringLiteral( "layer_%1" ).arg( outputLayer );
       component.mapLayerId = job->currentLayerId();
       component.opacity = job->currentLayerOpacity();
       component.compositionMode = job->currentLayerCompositionMode();
-      component.sourcePdfPath = mGeoPdfExporter->generateTemporaryFilepath( QStringLiteral( "layer_%1.pdf" ).arg( outputLayer ) );
+      component.sourcePdfPath = mGeospatialPdfExporter->generateTemporaryFilepath( QStringLiteral( "layer_%1.pdf" ).arg( outputLayer ) );
       pdfComponents << component;
 
       QPdfWriter pdfWriter( component.sourcePdfPath );
@@ -211,7 +211,7 @@ bool QgsMapRendererTask::run()
       outputLayer++;
       job->nextPart();
     }
-    QgsAbstractGeoPdfExporter::ExportDetails exportDetails = mGeoPdfExportDetails;
+    QgsAbstractGeospatialPdfExporter::ExportDetails exportDetails = mGeospatialPdfExportDetails;
     const double pageWidthMM = mMapSettings.outputSize().width() * 25.4 / mMapSettings.outputDpi();
     const double pageHeightMM = mMapSettings.outputSize().height() * 25.4 / mMapSettings.outputDpi();
     exportDetails.pageSizeMm = QSizeF( pageWidthMM, pageHeightMM );
@@ -223,19 +223,19 @@ bool QgsMapRendererTask::run()
     if ( mSaveWorldFile )
     {
       // setup georeferencing
-      QgsAbstractGeoPdfExporter::GeoReferencedSection georef;
+      QgsAbstractGeospatialPdfExporter::GeoReferencedSection georef;
       georef.crs = mMapSettings.destinationCrs();
       georef.pageBoundsMm = QgsRectangle( 0, 0, pageWidthMM, pageHeightMM );
       georef.controlPoints.reserve( 4 );
-      georef.controlPoints << QgsAbstractGeoPdfExporter::ControlPoint( QgsPointXY( 0, 0 ), mMapSettings.mapToPixel().toMapCoordinates( 0, 0 ) );
-      georef.controlPoints << QgsAbstractGeoPdfExporter::ControlPoint( QgsPointXY( pageWidthMM, 0 ), mMapSettings.mapToPixel().toMapCoordinates( mMapSettings.outputSize().width(), 0 ) );
-      georef.controlPoints << QgsAbstractGeoPdfExporter::ControlPoint( QgsPointXY( pageWidthMM, pageHeightMM ), mMapSettings.mapToPixel().toMapCoordinates( mMapSettings.outputSize().width(), mMapSettings.outputSize().height() ) );
-      georef.controlPoints << QgsAbstractGeoPdfExporter::ControlPoint( QgsPointXY( 0, pageHeightMM ), mMapSettings.mapToPixel().toMapCoordinates( 0, mMapSettings.outputSize().height() ) );
+      georef.controlPoints << QgsAbstractGeospatialPdfExporter::ControlPoint( QgsPointXY( 0, 0 ), mMapSettings.mapToPixel().toMapCoordinates( 0, 0 ) );
+      georef.controlPoints << QgsAbstractGeospatialPdfExporter::ControlPoint( QgsPointXY( pageWidthMM, 0 ), mMapSettings.mapToPixel().toMapCoordinates( mMapSettings.outputSize().width(), 0 ) );
+      georef.controlPoints << QgsAbstractGeospatialPdfExporter::ControlPoint( QgsPointXY( pageWidthMM, pageHeightMM ), mMapSettings.mapToPixel().toMapCoordinates( mMapSettings.outputSize().width(), mMapSettings.outputSize().height() ) );
+      georef.controlPoints << QgsAbstractGeospatialPdfExporter::ControlPoint( QgsPointXY( 0, pageHeightMM ), mMapSettings.mapToPixel().toMapCoordinates( 0, mMapSettings.outputSize().height() ) );
       exportDetails.georeferencedSections << georef;
     }
 
-    const bool res = mGeoPdfExporter->finalize( pdfComponents, mFileName, exportDetails );
-    mGeoPdfExporter.reset();
+    const bool res = mGeospatialPdfExporter->finalize( pdfComponents, mFileName, exportDetails );
+    mGeospatialPdfExporter.reset();
     mTempPainter.reset();
     mPdfWriter.reset();
     return res;
@@ -331,10 +331,10 @@ bool QgsMapRendererTask::run()
           if ( mExportMetadata )
           {
             QString creationDateString;
-            const QDateTime creationDateTime = mGeoPdfExportDetails.creationDateTime;
+            const QDateTime creationDateTime = mGeospatialPdfExportDetails.creationDateTime;
             if ( creationDateTime.isValid() )
             {
-              creationDateString = QStringLiteral( "D:%1" ).arg( mGeoPdfExportDetails.creationDateTime.toString( QStringLiteral( "yyyyMMddHHmmss" ) ) );
+              creationDateString = QStringLiteral( "D:%1" ).arg( mGeospatialPdfExportDetails.creationDateTime.toString( QStringLiteral( "yyyyMMddHHmmss" ) ) );
               if ( creationDateTime.timeZone().isValid() )
               {
                 int offsetFromUtc = creationDateTime.timeZone().offsetFromUtc( creationDateTime );
@@ -347,14 +347,14 @@ bool QgsMapRendererTask::run()
             }
             GDALSetMetadataItem( outputDS.get(), "CREATION_DATE", creationDateString.toUtf8().constData(), nullptr );
 
-            GDALSetMetadataItem( outputDS.get(), "AUTHOR", mGeoPdfExportDetails.author.toUtf8().constData(), nullptr );
+            GDALSetMetadataItem( outputDS.get(), "AUTHOR", mGeospatialPdfExportDetails.author.toUtf8().constData(), nullptr );
             const QString creator = QStringLiteral( "QGIS %1" ).arg( Qgis::version() );
             GDALSetMetadataItem( outputDS.get(), "CREATOR", creator.toUtf8().constData(), nullptr );
             GDALSetMetadataItem( outputDS.get(), "PRODUCER", creator.toUtf8().constData(), nullptr );
-            GDALSetMetadataItem( outputDS.get(), "SUBJECT", mGeoPdfExportDetails.subject.toUtf8().constData(), nullptr );
-            GDALSetMetadataItem( outputDS.get(), "TITLE", mGeoPdfExportDetails.title.toUtf8().constData(), nullptr );
+            GDALSetMetadataItem( outputDS.get(), "SUBJECT", mGeospatialPdfExportDetails.subject.toUtf8().constData(), nullptr );
+            GDALSetMetadataItem( outputDS.get(), "TITLE", mGeospatialPdfExportDetails.title.toUtf8().constData(), nullptr );
 
-            const QgsAbstractMetadataBase::KeywordMap keywords = mGeoPdfExportDetails.keywords;
+            const QgsAbstractMetadataBase::KeywordMap keywords = mGeospatialPdfExportDetails.keywords;
             QStringList allKeywords;
             for ( auto it = keywords.constBegin(); it != keywords.constEnd(); ++it )
             {
@@ -442,12 +442,12 @@ void QgsMapRendererTask::finished( bool result )
 
 void QgsMapRendererTask::prepare()
 {
-  if ( mGeoPDF )
+  if ( mGeospatialPDF )
   {
-    mGeoPdfExporter = std::make_unique< QgsMapRendererTaskGeoPdfExporter >( mMapSettings );
-    if ( mGeoPdfExportDetails.includeFeatures )
+    mGeospatialPdfExporter = std::make_unique< QgsMapRendererTaskGeospatialPdfExporter >( mMapSettings );
+    if ( mGeospatialPdfExportDetails.includeFeatures )
     {
-      mRenderedFeatureHandler = std::make_unique< QgsMapRendererTaskRenderedFeatureHandler >( static_cast< QgsMapRendererTaskGeoPdfExporter * >( mGeoPdfExporter.get() ), mMapSettings );
+      mRenderedFeatureHandler = std::make_unique< QgsMapRendererTaskRenderedFeatureHandler >( static_cast< QgsMapRendererTaskGeospatialPdfExporter * >( mGeospatialPdfExporter.get() ), mMapSettings );
       mMapSettings.addRenderedFeatureHandler( mRenderedFeatureHandler.get() );
     }
 
