@@ -48,6 +48,7 @@ class QgsMapLayerLegend;
 class QgsMapLayerRenderer;
 class QgsMapLayerStyleManager;
 class QgsProject;
+class QgsProviderMetadata;
 class QgsStyleEntityVisitorInterface;
 class QgsMapLayerTemporalProperties;
 class QgsMapLayerElevationProperties;
@@ -75,10 +76,13 @@ class CORE_EXPORT QgsMapLayer : public QObject
 {
     Q_OBJECT
 
+    Q_PROPERTY( QString id READ id WRITE setId NOTIFY idChanged )
     Q_PROPERTY( QString name READ name WRITE setName NOTIFY nameChanged )
     Q_PROPERTY( int autoRefreshInterval READ autoRefreshInterval WRITE setAutoRefreshInterval NOTIFY autoRefreshIntervalChanged )
     Q_PROPERTY( QgsLayerMetadata metadata READ metadata WRITE setMetadata NOTIFY metadataChanged )
     Q_PROPERTY( QgsCoordinateReferenceSystem crs READ crs WRITE setCrs NOTIFY crsChanged )
+    Q_PROPERTY( QgsCoordinateReferenceSystem verticalCrs READ verticalCrs WRITE setVerticalCrs NOTIFY verticalCrsChanged )
+    Q_PROPERTY( QgsCoordinateReferenceSystem crs3D READ crs3D NOTIFY crs3DChanged )
     Q_PROPERTY( Qgis::LayerType type READ type CONSTANT )
     Q_PROPERTY( bool isValid READ isValid NOTIFY isValidChanged )
     Q_PROPERTY( double opacity READ opacity WRITE setOpacity NOTIFY opacityChanged )
@@ -177,10 +181,10 @@ class CORE_EXPORT QgsMapLayer : public QObject
       CustomProperties   = 1 << 11, //!< Custom properties (by plugins for instance)
       GeometryOptions    = 1 << 12, //!< Geometry validation configuration
       Relations          = 1 << 13, //!< Relations
-      Temporal           = 1 << 14, //!< Temporal properties (since QGIS 3.14)
-      Legend             = 1 << 15, //!< Legend settings (since QGIS 3.16)
-      Elevation          = 1 << 16, //!< Elevation settings (since QGIS 3.18)
-      Notes              = 1 << 17, //!< Layer user notes (since QGIS 3.20)
+      Temporal           = 1 << 14, //!< Temporal properties \since QGIS 3.14
+      Legend             = 1 << 15, //!< Legend settings \since QGIS 3.16
+      Elevation          = 1 << 16, //!< Elevation settings \since QGIS 3.18
+      Notes              = 1 << 17, //!< Layer user notes \since QGIS 3.20
       AllStyleCategories = LayerConfiguration | Symbology | Symbology3D | Labeling | Fields | Forms | Actions |
                            MapTips | Diagrams | AttributeTable | Rendering | CustomProperties | GeometryOptions | Relations | Temporal | Legend | Elevation | Notes,
     };
@@ -198,10 +202,8 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     ~QgsMapLayer() override;
 
-    //! QgsMapLayer cannot be copied
-    QgsMapLayer( QgsMapLayer const & ) = delete;
-    //! QgsMapLayer cannot be copied
-    QgsMapLayer &operator=( QgsMapLayer const & ) = delete;
+    QgsMapLayer( const QgsMapLayer & ) = delete;
+    QgsMapLayer &operator=( const QgsMapLayer & ) = delete;
 
     /**
      * Returns a new instance equivalent to this one except for the id which
@@ -256,9 +258,31 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     static QString extensionPropertyType( PropertyType type );
 
-    //! Returns the layer's unique ID, which is used to access this layer from QgsProject.
+    /**
+     * Returns the layer's unique ID, which is used to access this layer from QgsProject.
+     *
+     * \see setId()
+     * \see idChanged()
+     */
     QString id() const;
 
+    /**
+     * Sets the layer's \a id.
+     *
+     * Returns TRUE if the layer ID was successfully changed, or FALSE if it could not be changed (e.g. because
+     * the layer is owned by a QgsProject or QgsMapLayerStore).
+     *
+     * \warning It is the caller's responsibility to ensure that the layer ID is unique in the desired context.
+     * Generally this method should not be called, and the auto generated ID should be used instead.
+     *
+     * \warning This method should not be called on layers which already belong to a QgsProject or QgsMapLayerStore.
+     *
+     * \see id()
+     * \see idChanged()
+     * \since QGIS 3.38
+     */
+    bool setId( const QString &id );
+
     /**
      * Set the display \a name of the layer.
      * \see name()
@@ -283,135 +307,122 @@ class CORE_EXPORT QgsMapLayer : public QObject
     virtual const QgsDataProvider *dataProvider() const SIP_SKIP;
 
     /**
-     * Sets the short name of the layer
-     *  used by QGIS Server to identify the layer.
-     * \see shortName()
+     * Returns the layer data provider's metadata, it may be NULLPTR.
+     * \since QGIS 3.40
      */
-    void setShortName( const QString &shortName ) { mShortName = shortName; }
+    QgsProviderMetadata *providerMetadata() const;
 
     /**
-     * Returns the short name of the layer
-     *  used by QGIS Server to identify the layer.
-     * \see setShortName()
+     * Sets the short name of the layer used by QGIS Server to identify the layer.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setShortName() instead.
      */
-    QString shortName() const;
+    Q_DECL_DEPRECATED void setShortName( const QString &shortName ) SIP_DEPRECATED;
 
     /**
-     * Sets the title of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     * \see title()
+     * Returns the short name of the layer used by QGIS Server to identify the layer.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->shortName() instead.
      */
-    void setTitle( const QString &title ) { mTitle = title; }
+    Q_DECL_DEPRECATED QString shortName() const SIP_DEPRECATED;
 
     /**
-     * Returns the title of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     * \returns the layer title
-     * \see setTitle()
+     * Sets the title of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setTitle() instead.
      */
-    QString title() const { return mTitle; }
+    Q_DECL_DEPRECATED void setTitle( const QString &title ) SIP_DEPRECATED;
 
     /**
-     * Sets the abstract of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     * \see abstract()
+     * Returns the title of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->title() instead.
      */
-    void setAbstract( const QString &abstract ) { mAbstract = abstract; }
+    Q_DECL_DEPRECATED QString title() const SIP_DEPRECATED;
 
     /**
-     * Returns the abstract of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     * \returns the layer abstract
-     * \see setAbstract()
+     * Sets the abstract of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setAbstract() instead.
      */
-    QString abstract() const { return mAbstract; }
+    Q_DECL_DEPRECATED void setAbstract( const QString &abstract ) SIP_DEPRECATED;
 
     /**
-     * Sets the keyword list of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     * \see keywordList()
+     * Returns the abstract of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->abstract() instead.
      */
-    void setKeywordList( const QString &keywords ) { mKeywordList = keywords; }
+    Q_DECL_DEPRECATED QString abstract() const SIP_DEPRECATED;
 
     /**
-     * Returns the keyword list of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     * \returns the layer keyword list
-     * \see setKeywordList()
+     * Sets the keyword list of the layerused by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setKeywordList() instead.
      */
-    QString keywordList() const { return mKeywordList; }
-
-    /* Layer dataUrl information */
+    Q_DECL_DEPRECATED void setKeywordList( const QString &keywords ) SIP_DEPRECATED;
 
     /**
-     * Sets the DataUrl of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  DataUrl is a a link to the underlying data represented by a particular layer.
-     * \see dataUrl()
+     * Returns the keyword list of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->keywordList() instead.
      */
-    void setDataUrl( const QString &dataUrl ) { mDataUrl = dataUrl; }
+    Q_DECL_DEPRECATED QString keywordList() const SIP_DEPRECATED;
 
     /**
-     * Returns the DataUrl of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  DataUrl is a a link to the underlying data represented by a particular layer.
-     * \returns the layer DataUrl
-     * \see setDataUrl()
+     * Sets the DataUrl of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setDataUrl() instead.
      */
-    QString dataUrl() const { return mDataUrl; }
+    Q_DECL_DEPRECATED void setDataUrl( const QString &dataUrl ) SIP_DEPRECATED;
 
     /**
-     * Sets the DataUrl format of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  DataUrl is a a link to the underlying data represented by a particular layer.
-     * \see dataUrlFormat()
+     * Returns the DataUrl of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->dataUrl() instead.
      */
-    void setDataUrlFormat( const QString &dataUrlFormat ) { mDataUrlFormat = dataUrlFormat; }
+    Q_DECL_DEPRECATED QString dataUrl() const SIP_DEPRECATED;
 
     /**
-     * Returns the DataUrl format of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  DataUrl is a a link to the underlying data represented by a particular layer.
-     * \returns the layer DataUrl format
-     * \see setDataUrlFormat()
+     * Sets the DataUrl format of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setDataUrlFormat() instead.
      */
-    QString dataUrlFormat() const { return mDataUrlFormat; }
-
-    /* Layer attribution information */
+    Q_DECL_DEPRECATED void setDataUrlFormat( const QString &dataUrlFormat ) SIP_DEPRECATED;
 
     /**
-     * Sets the attribution of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  Attribution indicates the provider of a layer or collection of layers.
-     * \see attribution()
+     * Returns the DataUrl format of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->dataUrlFormat() instead.
      */
-    void setAttribution( const QString &attrib ) { mAttribution = attrib; }
+    Q_DECL_DEPRECATED QString dataUrlFormat() const SIP_DEPRECATED;
 
     /**
-     * Returns the attribution of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  Attribution indicates the provider of a layer or collection of layers.
-     * \returns the layer attribution
-     * \see setAttribution()
+     * Sets the attribution of the layerused by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setAttribution() instead.
      */
-    QString attribution() const { return mAttribution; }
+    Q_DECL_DEPRECATED void setAttribution( const QString &attrib ) SIP_DEPRECATED;
 
     /**
-     * Sets the attribution URL of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  Attribution indicates the provider of a layer or collection of layers.
-     * \see attributionUrl()
+     * Returns the attribution of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->attribution() instead.
      */
-    void setAttributionUrl( const QString &attribUrl ) { mAttributionUrl = attribUrl; }
+    Q_DECL_DEPRECATED QString attribution() const SIP_DEPRECATED;
 
     /**
-     * Returns the attribution URL of the layer
-     *  used by QGIS Server in GetCapabilities request.
-     *  Attribution indicates the provider of a layer or collection of layers.
-     * \returns the layer attribution URL
-     * \see setAttributionUrl()
+     * Sets the attribution URL of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->setAttributionUrl() instead.
      */
-    QString attributionUrl() const { return mAttributionUrl; }
+    Q_DECL_DEPRECATED void setAttributionUrl( const QString &attribUrl ) SIP_DEPRECATED;
+
+    /**
+     * Returns the attribution URL of the layer used by QGIS Server in GetCapabilities request.
+     *
+     * \deprecated QGIS 3.38. Use serverProperties()->attributionUrl() instead.
+     */
+    Q_DECL_DEPRECATED QString attributionUrl() const SIP_DEPRECATED;
 
     /* Layer metadataUrl information */
 
@@ -433,7 +444,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *  MetadataUrl is a a link to the detailed, standardized metadata about the data.
      *  Since QGIS 3.22, it edits the first metadata URL link.
      * \see serverProperties()
-     * \deprecated since QGIS 3.22
+     * \deprecated QGIS 3.22
      */
     Q_DECL_DEPRECATED void setMetadataUrl( const QString &metaUrl ) SIP_DEPRECATED;
 
@@ -444,7 +455,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Since QGIS 3.22, it returns the first metadata URL link.
      * \returns the layer metadata URL
      * \see serverProperties()
-     * \deprecated since QGIS 3.22
+     * \deprecated QGIS 3.22
      */
     Q_DECL_DEPRECATED QString metadataUrl() const SIP_DEPRECATED;
 
@@ -454,7 +465,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *  MetadataUrlType indicates the standard to which the metadata complies.
      *  Since QGIS 3.22, it edits the first metadata URL type.
      * \see serverProperties()
-     * \deprecated since QGIS 3.22
+     * \deprecated QGIS 3.22
      */
     Q_DECL_DEPRECATED void setMetadataUrlType( const QString &metaUrlType ) SIP_DEPRECATED;
 
@@ -465,7 +476,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Since QGIS 3.22, it returns the first metadata URL type.
      * \returns the layer metadata type
      * \see serverProperties()
-     * \deprecated since QGIS 3.22
+     * \deprecated QGIS 3.22
      */
     Q_DECL_DEPRECATED QString metadataUrlType() const SIP_DEPRECATED;
 
@@ -475,7 +486,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      *  MetadataUrlType indicates how the metadata is structured.
      *  Since QGIS 3.22, it edits the first metadata URL format.
      * \see serverProperties()
-     * \deprecated since QGIS 3.22
+     * \deprecated QGIS 3.22
      */
     Q_DECL_DEPRECATED void setMetadataUrlFormat( const QString &metaUrlFormat ) SIP_DEPRECATED;
 
@@ -486,7 +497,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Since QGIS 3.22, it returns the first metadata URL format.
      * \returns the layer metadata format
      * \see serverProperties()
-     * \deprecated since QGIS 3.22
+     * \deprecated QGIS 3.22
      */
     Q_DECL_DEPRECATED QString metadataUrlFormat() const SIP_DEPRECATED;
 
@@ -638,7 +649,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     enum ReadFlag SIP_ENUM_BASETYPE( IntFlag )
     {
       FlagDontResolveLayers = 1 << 0, //!< Don't resolve layer paths or create data providers for layers.
-      FlagTrustLayerMetadata = 1 << 1, //!< Trust layer metadata. Improves layer load time by skipping expensive checks like primary key unicity, geometry type and srid and by using estimated metadata on layer load. Since QGIS 3.16
+      FlagTrustLayerMetadata = 1 << 1, //!< Trust layer metadata. Improves layer load time by skipping expensive checks like primary key unicity, geometry type and srid and by using estimated metadata on layer load \since QGIS 3.16
       FlagReadExtentFromXml = 1 << 2, //!< Read extent from xml and skip get extent from provider.
       FlagForceReadOnly = 1 << 3, //!< Force open as read only.
     };
@@ -646,13 +657,12 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Sets state from DOM document
-     * \param layerElement The DOM element corresponding to ``maplayer'' tag
+     * \param layerElement The DOM element corresponding to ``maplayer`` tag
      * \param context writing context (e.g. for conversion between relative and absolute paths)
      * \param flags optional argument which can be used to control layer reading behavior.
      * \param preloadedProvider optional preloaded data provider that will be used as data provider for this layer, takes ownership (since QGIS 3.32)
-     * \note
      *
-     * The DOM node corresponds to a DOM document project file XML element read
+     * \note The DOM node corresponds to a DOM document project file XML element read
      * by QgsProject.
      *
      * This, in turn, calls readXml() (and then readSymbology()), which is overridable
@@ -667,12 +677,10 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Stores state in DOM node
-     * \param layerElement is a DOM element corresponding to ``maplayer'' tag
+     * \param layerElement is a DOM element corresponding to ``maplayer`` tag
      * \param document is a the DOM document being written
      * \param context reading context (e.g. for conversion between relative and absolute paths)
-     * \note
-     *
-     * The DOM node corresponds to a DOM document project file XML element to be
+     * \note The DOM node corresponds to a DOM document project file XML element to be
      * written by QgsProject.
      *
      * This, in turn, calls writeXml() (and then writeSymbology), which is over-rideable
@@ -766,15 +774,23 @@ class CORE_EXPORT QgsMapLayer : public QObject
                                       QString &msgError SIP_OUT,
                                       QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories );
 
+
+    // TODO QGIS 4.0 -- fix this. We incorrectly have a single boolean flag which in which false is used inconsistently for "a style WAS found but an error occurred loading it" vs "no style was found".
+    // The first (style found, error occurred loading it) should trigger a user-facing warning, whereas the second (no style found) isn't reflective of an error at all.
+
     /**
      * Loads a named style from file/local db/datasource db
      * \param theURI the URI of the style or the URI of the layer
      * \param resultFlag will be set to TRUE if a named style is correctly loaded
      * \param loadFromLocalDb if TRUE forces to load from local db instead of datasource one
      * \param categories the style categories to be loaded.
+     * \param flags flags controlling how the style should be loaded (since QGIS 3.38)
+     *
+     * \returns status message, which may indicate success or contain an error message
      */
     virtual QString loadNamedStyle( const QString &theURI, bool &resultFlag SIP_OUT, bool loadFromLocalDb,
-                                    QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories );
+                                    QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories,
+                                    Qgis::LoadStyleFlags flags = Qgis::LoadStyleFlags() );
 
 #ifndef SIP_RUN
 
@@ -962,12 +978,90 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Returns the layer's spatial reference system.
+     *
+     * \warning Since QGIS 3.38, consider using crs3D() whenever transforming 3D data or whenever
+     * z/elevation value handling is important.
+     *
+     * \see setCrs()
+     * \see crs3D()
+     * \see verticalCrs()
+     * \see crsChanged()
      */
     QgsCoordinateReferenceSystem crs() const;
 
-    //! Sets layer's spatial reference system
+    /**
+     * Returns the layer's vertical coordinate reference system.
+     *
+     * If the layer crs() is a compound CRS, then the CRS returned will
+     * be the vertical component of crs(). Otherwise it will be the value
+     * explicitly set by a call to setVerticalCrs().
+     *
+     * The returned CRS will be invalid if the layer has no vertical CRS.
+     *
+     * \note Consider also using crs3D(), which will return a CRS which takes into account
+     * both crs() and verticalCrs().
+     *
+     * \see crs()
+     * \see crs3D()
+     * \see setVerticalCrs()
+     *
+     * \since QGIS 3.38
+     */
+    QgsCoordinateReferenceSystem verticalCrs() const;
+
+    /**
+     * Returns the CRS to use for the layer when transforming 3D data, or when z/elevation
+     * value handling is important.
+     *
+     * The returned CRS will take into account verticalCrs() when appropriate, e.g. it may return a compound
+     * CRS consisting of crs() + verticalCrs(). This method may still return a 2D CRS, e.g in the
+     * case that crs() is a 2D CRS and no verticalCrs() has been set for the layer. Check QgsCoordinateReferenceSystem::type()
+     * on the returned value to determine the type of CRS returned by this method.
+     *
+     * \warning It is NOT guaranteed that the returned CRS will actually be a 3D CRS, but rather
+     * it is guaranteed that the returned CRS is ALWAYS the most appropriate CRS to use when handling 3D data.
+     *
+     * \see crs()
+     * \see verticalCrs()
+     * \see crs3DChanged()
+     *
+     * \since QGIS 3.38
+     */
+    QgsCoordinateReferenceSystem crs3D() const;
+
+    /**
+     * Sets layer's spatial reference system.
+     *
+     * If \a emitSignal is TRUE, changing the CRS will trigger a crsChanged() signal. Additionally, if \a crs is a compound
+     * CRS, then the verticalCrsChanged() signal will also be emitted.
+     *
+     * \see crs()
+     * \see crsChanged()
+     * \see setVerticalCrs()
+     */
     void setCrs( const QgsCoordinateReferenceSystem &srs, bool emitSignal = true );
 
+    /**
+     * Sets the layer's vertical coordinate reference system.
+     *
+     * The verticalCrsChanged() signal will be raised if the vertical CRS is changed.
+     *
+     * \note If the layer crs() is a compound CRS, then the CRS returned for
+     * verticalCrs() will be the vertical component of crs(). Otherwise it will be the value
+     * explicitly set by this call.
+     *
+     * \param crs the vertical CRS
+     * \param errorMessage will be set to a descriptive message if the vertical CRS could not be set
+     *
+     * \returns TRUE if vertical CRS was successfully set
+     *
+     * \see verticalCrs()
+     * \see setCrs()
+     *
+     * \since QGIS 3.38
+     */
+    bool setVerticalCrs( const QgsCoordinateReferenceSystem &crs, QString *errorMessage SIP_OUT = nullptr );
+
     /**
      * Returns the layer data provider coordinate transform context
      * or a default transform context if the layer does not have a valid data provider.
@@ -1022,11 +1116,14 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     QString saveNamedMetadata( const QString &uri, bool &resultFlag );
 
+    // TODO QGIS 4.0 -- fix this. We incorrectly have a single boolean flag which in which false is used inconsistently for "metadata WAS found but an error occurred loading it" vs "no metadata was found".
+    // The first (metadata found, error occurred loading it) should trigger a user-facing warning, whereas the second (no metadata found) isn't reflective of an error at all.
+
     /**
      * Retrieve a named metadata for this layer if one
      * exists (either as a .qmd file on disk or as a
      * record in the users style table in their personal qgis.db)
-     * \param uri - the file name or other URI for the
+     * \param uri the file name or other URI for the
      * metadata file. First an attempt will be made to see if this
      * is a file and load that, if that fails the qgis.db metadata
      * table will be consulted to see if there is a metadata who's
@@ -1037,6 +1134,9 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     virtual QString loadNamedMetadata( const QString &uri, bool &resultFlag SIP_OUT );
 
+    // TODO QGIS 4.0 -- fix this. We incorrectly have a single boolean flag which in which false is used inconsistently for "metadata WAS found but an error occurred loading it" vs "no metadata was found".
+    // The first (metadata found, error occurred loading it) should trigger a user-facing warning, whereas the second (no metadata found) isn't reflective of an error at all.
+
     /**
      * Retrieve the default metadata for this layer if one
      * exists (either as a .qmd file on disk or as a
@@ -1074,6 +1174,9 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     virtual QString styleURI() const;
 
+    // TODO QGIS 4.0 -- fix this. We incorrectly have a single boolean flag which in which false is used inconsistently for "a style WAS found but an error occurred loading it" vs "no style was found".
+    // The first (style found, error occurred loading it) should trigger a user-facing warning, whereas the second (no style found) isn't reflective of an error at all.
+
     /**
      * Retrieve the default style for this layer if one
      * exists (either as a .qml file on disk or as a
@@ -1085,11 +1188,14 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     virtual QString loadDefaultStyle( bool &resultFlag SIP_OUT );
 
+    // TODO QGIS 4.0 -- fix this. We incorrectly have a single boolean flag which in which false is used inconsistently for "a style WAS found but an error occurred loading it" vs "no style was found".
+    // The first (style found, error occurred loading it) should trigger a user-facing warning, whereas the second (no style found) isn't reflective of an error at all.
+
     /**
      * Retrieve a named style for this layer if one
      * exists (either as a .qml file on disk or as a
      * record in the users style table in their personal qgis.db)
-     * \param uri - the file name or other URI for the
+     * \param uri the file name or other URI for the
      * style file. First an attempt will be made to see if this
      * is a file and load that, if that fails the qgis.db styles
      * table will be consulted to see if there is a style who's
@@ -1097,10 +1203,11 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \param resultFlag a reference to a flag that will be set to FALSE if
      * we did not manage to load the default style.
      * \param categories the style categories to be loaded.
+     * \param flags flags controlling how the style should be loaded (since QGIS 3.38)
      * \returns a QString with any status messages
      * \see loadDefaultStyle()
      */
-    virtual QString loadNamedStyle( const QString &uri, bool &resultFlag SIP_OUT, QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories );
+    virtual QString loadNamedStyle( const QString &uri, bool &resultFlag SIP_OUT, QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories, Qgis::LoadStyleFlags flags = Qgis::LoadStyleFlags() );
 
     /**
      * Retrieve a named style for this layer from a sqlite database.
@@ -1175,7 +1282,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \returns a QString with any status messages
      * \see loadNamedStyle()
      * \see saveNamedStyle()
-     * \deprecated since QGIS 3.26
+     * \deprecated QGIS 3.26
      */
     Q_DECL_DEPRECATED virtual QString saveDefaultStyle( bool &resultFlag SIP_OUT ) SIP_DEPRECATED;
 
@@ -1292,11 +1399,12 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * the specified data \a provider in use. See QgsDataSourceUri and the documentation for the various QgsMapLayer
      * subclasses for further details on data source strings.
      *
-     * The \a baseName argument specifies the user-visible name to use for the layer. (See name() or setName()).
+     * The \a baseName argument specifies the user-visible name to use for the layer. (See name() or setName()). If not
+     * specified, then the current name will be left unchanged (since QGIS 3.40).
      *
      * The \a provider argument is used to specify the unique key of the data provider to use for
      * the layer. This must match one of the values returned by QgsProviderRegistry::instance()->providerList().
-     * (See providerType()).
+     * (See providerType()). If not specified, then the current data provider will be used (since QGIS 3.40).
      *
      * If \a loadDefaultStyleFlag is set to TRUE then the layer's existing style will be reset to the default
      * for the data source.
@@ -1314,7 +1422,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \see dataSourceChanged()
      * \since QGIS 3.20
      */
-    void setDataSource( const QString &dataSource, const QString &baseName, const QString &provider, bool loadDefaultStyleFlag = false );
+    void setDataSource( const QString &dataSource, const QString &baseName = QString(), const QString &provider = QString(), bool loadDefaultStyleFlag = false );
 
     /**
      * Updates the data source of the layer.
@@ -1376,7 +1484,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \see dataSourceChanged()
      * \since QGIS 3.20
      */
-    void setDataSource( const QString &dataSource, const QString &baseName, const QString &provider, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags );
+    void setDataSource( const QString &dataSource, const QString &baseName, const QString &provider, const QgsDataProvider::ProviderOptions &options, Qgis::DataProviderReadFlags flags );
 
     /**
      * Returns the provider type (provider key) for this layer
@@ -1485,12 +1593,12 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Returns TRUE if auto refresh is enabled for the layer.
      * \see autoRefreshInterval()
      * \see setAutoRefreshEnabled()
-     * \deprecated use autoRefreshMode() instead.
+     * \deprecated QGIS 3.40. Use autoRefreshMode() instead.
      */
     Q_DECL_DEPRECATED bool hasAutoRefreshEnabled() const SIP_DEPRECATED;
 
     /**
-     * Returns the layer's automatical refresh mode.
+     * Returns the layer's automatic refresh mode.
      * \see autoRefreshInterval()
      * \see setAutoRefreshMode()
      * \since QGIS 3.34
@@ -1521,7 +1629,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * Sets whether auto refresh is enabled for the layer.
      * \see hasAutoRefreshEnabled()
      * \see setAutoRefreshInterval()
-     * \deprecated Use setAutoRefreshMode() instead.
+     * \deprecated QGIS 3.40. Use setAutoRefreshMode() instead.
      */
     Q_DECL_DEPRECATED void setAutoRefreshEnabled( bool enabled ) SIP_DEPRECATED;
 
@@ -1634,7 +1742,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Returns path to the placeholder image or an empty string if a generated legend is shown
-     * \return placholder image path
+     * \return placeholder image path
      * \since QGIS 3.22
      */
     QString legendPlaceholderImage() const { return mLegendPlaceholderImage;}
@@ -1688,12 +1796,12 @@ class CORE_EXPORT QgsMapLayer : public QObject
 
     /**
      * Returns provider read flag deduced from layer read flags \a layerReadFlags and a dom node \a layerNode
-     * that describes a layer (corresponding to ``maplayer'' tag in a DOM document project file read by QgsProject).
+     * that describes a layer (corresponding to ``maplayer`` tag in a DOM document project file read by QgsProject).
      * This static method is used when loading a project.
      *
      * \since QGIS 3.32
      */
-    static QgsDataProvider::ReadFlags providerReadFlags( const QDomNode &layerNode, QgsMapLayer::ReadFlags layerReadFlags );
+    static Qgis::DataProviderReadFlags providerReadFlags( const QDomNode &layerNode, QgsMapLayer::ReadFlags layerReadFlags );
 
   public slots:
 
@@ -1811,14 +1919,58 @@ class CORE_EXPORT QgsMapLayer : public QObject
     void statusChanged( const QString &status );
 
     /**
-     * Emitted when the name has been changed
+     * Emitted when the layer's ID has been changed.
      *
+     * \see id()
+     * \see setId()
+     *
+     * \since QGIS 3.38
+     */
+    void idChanged( const QString &id );
+
+    /**
+     * Emitted when the name has been changed
      */
     void nameChanged();
 
-    //! Emit a signal that layer's CRS has been reset
+    /**
+     * Emitted when the crs() of the layer has changed.
+     *
+     * \see crs()
+     * \see setCrs()
+     * \see verticalCrsChanged()
+     * \see crs3DChanged()
+     */
     void crsChanged();
 
+    /**
+     * Emitted when the crs3D() of the layer has changed.
+     *
+     * \see crs3D()
+     * \see crsChanged()
+     * \see verticalCrsChanged()
+     *
+     * \since QGIS 3.38
+     */
+    void crs3DChanged();
+
+    /**
+     * Emitted when the verticalCrs() of the layer has changed.
+     *
+     * This signal will be emitted whenever the vertical CRS of the layer is changed, either
+     * as a direct result of a call to setVerticalCrs() or when setCrs() is called with a compound
+     * CRS.
+     *
+     * \see crsChanged()
+     * \see crs3DChanged()
+     * \see setCrs()
+     * \see setVerticalCrs()
+     * \see verticalCrs()
+     *
+     * \since QGIS 3.38
+     */
+    void verticalCrsChanged();
+
     /**
      * By emitting this signal the layer tells that either appearance or content have been changed
      * and any view showing the rendered layer should refresh itself.
@@ -2002,7 +2154,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \see dataSourceChanged()
      * \since QGIS 3.20
      */
-    virtual void setDataSourcePrivate( const QString &dataSource, const QString &baseName, const QString &provider, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags );
+    virtual void setDataSourcePrivate( const QString &dataSource, const QString &baseName, const QString &provider, const QgsDataProvider::ProviderOptions &options, Qgis::DataProviderReadFlags flags );
 
   protected:
 
@@ -2123,21 +2275,6 @@ class CORE_EXPORT QgsMapLayer : public QObject
     //! Name of the layer - used for display
     QString mLayerName;
 
-    QString mShortName;
-    QString mTitle;
-
-    //! Description of the layer
-    QString mAbstract;
-    QString mKeywordList;
-
-    //! DataUrl of the layer
-    QString mDataUrl;
-    QString mDataUrlFormat;
-
-    //! Attribution of the layer
-    QString mAttribution;
-    QString mAttributionUrl;
-
     //! WMS legend
     QString mLegendUrl;
     QString mLegendUrlFormat;
@@ -2151,7 +2288,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Checks whether a new set of dependencies will introduce a cycle
      * this method is now deprecated and always return FALSE, because circular dependencies are now correctly managed.
-     * \deprecated since QGIS 3.10
+     * \deprecated QGIS 3.10
      */
     Q_DECL_DEPRECATED bool hasDependencyCycle( const QSet<QgsMapLayerDependency> & ) const {return false;}
 
@@ -2203,7 +2340,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
 #ifndef SIP_RUN
 
     /**
-     * Returns an HTML fragment containing general  metadata information, for use
+     * Returns an HTML fragment containing general metadata information, for use
      * in the htmlMetadata() method.
      *
      * \note Not available in Python bindings.
@@ -2211,6 +2348,16 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * \since QGIS 3.22
      */
     QString generalHtmlMetadata() const;
+
+    /**
+     * Returns an HTML fragment containing custom property information, for use
+     * in the htmlMetadata() method.
+     *
+     * \note Not available in Python bindings.
+     *
+     * \since QGIS 3.40
+     */
+    QString customPropertyHtmlMetadata() const;
 #endif
 
 #ifndef SIP_RUN
@@ -2231,13 +2378,15 @@ class CORE_EXPORT QgsMapLayer : public QObject
     QString saveNamedProperty( const QString &uri, QgsMapLayer::PropertyType type,
                                bool &resultFlag, StyleCategories categories = AllStyleCategories );
     QString loadNamedProperty( const QString &uri, QgsMapLayer::PropertyType type,
-                               bool &resultFlag, StyleCategories categories = AllStyleCategories );
+                               bool &namedPropertyExists, bool &propertySuccessfullyLoaded, StyleCategories categories = AllStyleCategories, Qgis::LoadStyleFlags flags = Qgis::LoadStyleFlags() );
     bool loadNamedPropertyFromDatabase( const QString &db, const QString &uri, QString &xml, QgsMapLayer::PropertyType type );
 
     // const method because extents are mutable
     void updateExtent( const QgsRectangle &extent ) const;
     void updateExtent( const QgsBox3D &extent ) const;
 
+    bool rebuildCrs3D( QString *error = nullptr );
+
     /**
      * This method returns TRUE by default but can be overwritten to specify
      * that a certain layer is writable.
@@ -2249,6 +2398,8 @@ class CORE_EXPORT QgsMapLayer : public QObject
      * private to make sure setCrs must be used and crsChanged() is emitted.
     */
     QgsCoordinateReferenceSystem mCRS;
+    QgsCoordinateReferenceSystem mVerticalCrs;
+    QgsCoordinateReferenceSystem mCrs3D;
 
     //! Unique ID of this layer - used to refer to this layer in map layer registry
     QString mID;
