@@ -203,7 +203,7 @@ void QgsMapLayerLegendUtils::setLegendNodeCustomSymbol( QgsLayerTreeLayer *nodeL
   {
     QDomDocument doc;
     QgsReadWriteContext rwContext;
-    rwContext.setPathResolver( QgsProject::instance()->pathResolver() );
+    rwContext.setPathResolver( QgsProject::instance()->pathResolver() ); // skip-keyword-check
     const QDomElement elem = QgsSymbolLayerUtils::saveSymbol( QStringLiteral( "custom symbol" ), symbol, doc, rwContext );
     doc.appendChild( elem );
     nodeLayer->setCustomProperty( "legend/custom-symbol-" + QString::number( originalIndex ), doc.toString() );
@@ -223,7 +223,7 @@ QgsSymbol *QgsMapLayerLegendUtils::legendNodeCustomSymbol( QgsLayerTreeLayer *no
   const QDomElement elem = doc.documentElement();
 
   QgsReadWriteContext rwContext;
-  rwContext.setPathResolver( QgsProject::instance()->pathResolver() );
+  rwContext.setPathResolver( QgsProject::instance()->pathResolver() ); // skip-keyword-check
 
   return QgsSymbolLayerUtils::loadSymbol( elem, rwContext );
 }
@@ -234,7 +234,7 @@ void QgsMapLayerLegendUtils::setLegendNodeColorRampSettings( QgsLayerTreeLayer *
   {
     QDomDocument doc;
     QgsReadWriteContext rwContext;
-    rwContext.setPathResolver( QgsProject::instance()->pathResolver() );
+    rwContext.setPathResolver( QgsProject::instance()->pathResolver() ); // skip-keyword-check
     QDomElement elem = doc.createElement( QStringLiteral( "rampSettings" ) );
     settings->writeXml( doc, elem, rwContext );
     doc.appendChild( elem );
@@ -255,7 +255,7 @@ QgsColorRampLegendNodeSettings *QgsMapLayerLegendUtils::legendNodeColorRampSetti
   const QDomElement elem = doc.documentElement();
 
   QgsReadWriteContext rwContext;
-  rwContext.setPathResolver( QgsProject::instance()->pathResolver() );
+  rwContext.setPathResolver( QgsProject::instance()->pathResolver() ); // skip-keyword-check
 
   QgsColorRampLegendNodeSettings settings;
   settings.readXml( elem, rwContext );
@@ -552,16 +552,17 @@ QList<QgsLayerTreeModelLegendNode *> QgsDefaultMeshLayerLegend::createLayerTreeM
     return nodes;
   }
 
-  nodes << new QgsSimpleLegendNode( nodeLayer, name );
-
   if ( indexScalar > -1 )
   {
+    const QString scalarNameKey = QStringLiteral( "scalarName" );
+    nodes << new QgsSimpleLegendNode( nodeLayer, mLayer->datasetGroupMetadata( indexScalar ).name(),
+                                      QIcon(), nullptr, scalarNameKey );
     const QgsMeshRendererScalarSettings settings = rendererSettings.scalarSettings( indexScalar );
     const QgsColorRampShader shader = settings.colorRampShader();
     switch ( shader.colorRampType() )
     {
       case Qgis::ShaderInterpolationMethod::Linear:
-        if ( ! shader.legendSettings() || shader.legendSettings()->useContinuousLegend() )
+        if ( !shader.legendSettings() || shader.legendSettings()->useContinuousLegend() )
         {
           // for interpolated shaders we use a ramp legend node
           if ( !shader.colorRampItemList().isEmpty() )
@@ -569,7 +570,10 @@ QList<QgsLayerTreeModelLegendNode *> QgsDefaultMeshLayerLegend::createLayerTreeM
             nodes << new QgsColorRampLegendNode( nodeLayer, shader.createColorRamp(),
                                                  shader.legendSettings() ? *shader.legendSettings() : QgsColorRampLegendNodeSettings(),
                                                  shader.minimumValue(),
-                                                 shader.maximumValue() );
+                                                 shader.maximumValue(),
+                                                 nullptr,
+                                                 QStringLiteral( "scalarLegend" ),
+                                                 scalarNameKey );
           }
           break;
         }
@@ -582,13 +586,76 @@ QList<QgsLayerTreeModelLegendNode *> QgsDefaultMeshLayerLegend::createLayerTreeM
         settings.colorRampShader().legendSymbologyItems( items );
         for ( const QPair< QString, QColor > &item : items )
         {
-          nodes << new QgsRasterSymbolLegendNode( nodeLayer, item.second, item.first );
+          nodes << new QgsRasterSymbolLegendNode( nodeLayer, item.second, item.first, nullptr, false,
+                                                  QStringLiteral( "scalarLegend" ) + QUuid::createUuid().toString(),
+                                                  scalarNameKey
+                                                );
         }
         break;
       }
     }
   }
 
+  if ( indexVector > -1 )
+  {
+    const QgsMeshRendererVectorSettings settings = rendererSettings.vectorSettings( indexVector );
+    const QString vectorNameKey = QStringLiteral( "vectorName" );
+    switch ( settings.coloringMethod() )
+    {
+      case QgsInterpolatedLineColor::ColoringMethod::SingleColor:
+      {
+        const QColor arrowColor = settings.color();
+        const QIcon vectorIcon = QgsApplication::getThemeIcon( QStringLiteral( "/propertyicons/meshvectors.svg" ), arrowColor, arrowColor );
+        nodes << new QgsSimpleLegendNode( nodeLayer, mLayer->datasetGroupMetadata( indexVector ).name(),
+                                          vectorIcon, nullptr, vectorNameKey );
+        break;
+      }
+      case QgsInterpolatedLineColor::ColoringMethod::ColorRamp:
+      {
+        const QIcon vectorIcon = QgsApplication::getThemeIcon( QStringLiteral( "/propertyicons/meshvectors.svg" ) );
+        nodes << new QgsSimpleLegendNode( nodeLayer, mLayer->datasetGroupMetadata( indexVector ).name(),
+                                          vectorIcon, nullptr, vectorNameKey );
+        const QgsColorRampShader shader = settings.colorRampShader();
+        switch ( shader.colorRampType() )
+        {
+          case Qgis::ShaderInterpolationMethod::Linear:
+            if ( !shader.legendSettings() || shader.legendSettings()->useContinuousLegend() )
+            {
+              // for interpolated shaders we use a ramp legend node
+              if ( !shader.colorRampItemList().isEmpty() )
+              {
+                nodes << new QgsColorRampLegendNode( nodeLayer, shader.createColorRamp(),
+                                                     shader.legendSettings() ? *shader.legendSettings() : QgsColorRampLegendNodeSettings(),
+                                                     shader.minimumValue(),
+                                                     shader.maximumValue(),
+                                                     nullptr,
+                                                     QStringLiteral( "vectorLegend" ),
+                                                     vectorNameKey );
+              }
+              break;
+            }
+            [[fallthrough]];
+          case Qgis::ShaderInterpolationMethod::Discrete:
+          case Qgis::ShaderInterpolationMethod::Exact:
+          {
+            // for all others we use itemised lists
+            QgsLegendColorList items;
+            settings.colorRampShader().legendSymbologyItems( items );
+            for ( const QPair< QString, QColor > &item : items )
+            {
+              nodes << new QgsRasterSymbolLegendNode( nodeLayer, item.second, item.first, nullptr, false,
+                                                      QStringLiteral( "vectorLegend" ) + QUuid::createUuid().toString(),
+                                                      vectorNameKey
+                                                    );
+            }
+            break;
+          }
+          break;
+        }
+      }
+    }
+  }
+
   return nodes;
 }
 
