@@ -191,7 +191,7 @@ void QgsVectorLayerJoinBuffer::cacheJoinLayer( QgsVectorLayerJoinInfo &joinInfo
           // Check for name collisions
           int fieldIndex = mLayer->fields().indexFromName( joinFieldName );
           if ( fieldIndex >= 0
-               && mLayer->fields().fieldOrigin( fieldIndex ) != QgsFields::OriginJoin )
+               && mLayer->fields().fieldOrigin( fieldIndex ) != Qgis::FieldOrigin::Join )
             continue;
 
           attributesCache.append( attrs.at( i ) );
@@ -273,7 +273,7 @@ void QgsVectorLayerJoinBuffer::updateFields( QgsFields &fields )
       {
         QgsField f = joinFields.at( idx );
         f.setName( prefix + f.name() );
-        fields.append( f, QgsFields::OriginJoin, idx + ( joinIdx * 1000 ) );
+        fields.append( f, Qgis::FieldOrigin::Join, idx + ( joinIdx * 1000 ) );
       }
     }
   }
@@ -413,7 +413,7 @@ int QgsVectorLayerJoinBuffer::joinedFieldsOffset( const QgsVectorLayerJoinInfo *
 
   for ( int i = 0; i < fields.count(); ++i )
   {
-    if ( fields.fieldOrigin( i ) != QgsFields::OriginJoin )
+    if ( fields.fieldOrigin( i ) != Qgis::FieldOrigin::Join )
       continue;
 
     if ( fields.fieldOriginIndex( i ) / 1000 == joinIndex )
@@ -424,7 +424,7 @@ int QgsVectorLayerJoinBuffer::joinedFieldsOffset( const QgsVectorLayerJoinInfo *
 
 const QgsVectorLayerJoinInfo *QgsVectorLayerJoinBuffer::joinForFieldIndex( int index, const QgsFields &fields, int &sourceFieldIndex ) const
 {
-  if ( fields.fieldOrigin( index ) != QgsFields::OriginJoin )
+  if ( fields.fieldOrigin( index ) != Qgis::FieldOrigin::Join )
     return nullptr;
 
   int originIndex = fields.fieldOriginIndex( index );
@@ -645,7 +645,7 @@ bool QgsVectorLayerJoinBuffer::addFeatures( QgsFeatureList &features, QgsFeature
 
 bool QgsVectorLayerJoinBuffer::changeAttributeValue( QgsFeatureId fid, int field, const QVariant &newValue, const QVariant &oldValue )
 {
-  if ( mLayer->fields().fieldOrigin( field ) != QgsFields::OriginJoin )
+  if ( mLayer->fields().fieldOrigin( field ) != Qgis::FieldOrigin::Join )
     return false;
 
   int srcFieldIndex;
