@@ -23,13 +23,14 @@
 #include "qgsauxiliarystorage.h"
 #include "qgslinesymbol.h"
 #include "qgsfillsymbol.h"
+#include "qgsmarkersymbol.h"
 
 QgsExpressionContext QgsCalloutWidget::createExpressionContext() const
 {
   if ( auto *lExpressionContext = mContext.expressionContext() )
     return *lExpressionContext;
 
-  QgsExpressionContext expContext( mContext.globalProjectAtlasMapLayerScopes( vectorLayer() ) );
+  QgsExpressionContext expContext( mContext.globalProjectAtlasMapLayerScopes( layer() ) );
   QgsExpressionContextScope *symbolScope = QgsExpressionContextUtils::updateSymbolScope( nullptr, new QgsExpressionContextScope() );
   symbolScope->addVariable( QgsExpressionContextScope::StaticVariable( QgsExpressionContext::EXPR_SYMBOL_COLOR, QColor(), true ) );
   expContext << symbolScope;
@@ -72,7 +73,7 @@ QgsSymbolWidgetContext QgsCalloutWidget::context() const
 
 void QgsCalloutWidget::registerDataDefinedButton( QgsPropertyOverrideButton *button, QgsCallout::Property key )
 {
-  button->init( static_cast< int >( key ), callout()->dataDefinedProperties(), QgsCallout::propertyDefinitions(), mVectorLayer, true );
+  button->init( static_cast< int >( key ), callout()->dataDefinedProperties(), QgsCallout::propertyDefinitions(), qobject_cast< QgsVectorLayer * >( mLayer ), true );
   connect( button, &QgsPropertyOverrideButton::changed, this, &QgsCalloutWidget::updateDataDefinedProperty );
   connect( button, &QgsPropertyOverrideButton::createAuxiliaryField, this, &QgsCalloutWidget::createAuxiliaryField );
 
@@ -82,14 +83,18 @@ void QgsCalloutWidget::registerDataDefinedButton( QgsPropertyOverrideButton *but
 void QgsCalloutWidget::createAuxiliaryField()
 {
   // try to create an auxiliary layer if not yet created
-  if ( !mVectorLayer->auxiliaryLayer() )
+  QgsVectorLayer *vectorLayer = qobject_cast< QgsVectorLayer * >( mLayer );
+  if ( !vectorLayer )
+    return;
+
+  if ( !vectorLayer->auxiliaryLayer() )
   {
-    QgsNewAuxiliaryLayerDialog dlg( mVectorLayer, this );
+    QgsNewAuxiliaryLayerDialog dlg( vectorLayer, this );
     dlg.exec();
   }
 
   // return if still not exists
-  if ( !mVectorLayer->auxiliaryLayer() )
+  if ( !vectorLayer->auxiliaryLayer() )
     return;
 
   QgsPropertyOverrideButton *button = qobject_cast<QgsPropertyOverrideButton *>( sender() );
@@ -97,9 +102,9 @@ void QgsCalloutWidget::createAuxiliaryField()
   const QgsPropertyDefinition def = QgsCallout::propertyDefinitions()[static_cast< int >( key )];
 
   // create property in auxiliary storage if necessary
-  if ( !mVectorLayer->auxiliaryLayer()->exists( def ) )
+  if ( !vectorLayer->auxiliaryLayer()->exists( def ) )
   {
-    mVectorLayer->auxiliaryLayer()->addAuxiliaryField( def );
+    vectorLayer->auxiliaryLayer()->addAuxiliaryField( def );
   }
 
   // update property with join field name from auxiliary storage
@@ -128,7 +133,7 @@ void QgsCalloutWidget::updateDataDefinedProperty()
 // QgsSimpleLineCalloutWidget
 //
 
-QgsSimpleLineCalloutWidget::QgsSimpleLineCalloutWidget( QgsVectorLayer *vl, QWidget *parent )
+QgsSimpleLineCalloutWidget::QgsSimpleLineCalloutWidget( QgsMapLayer *vl, QWidget *parent )
   : QgsCalloutWidget( parent, vl )
 {
   setupUi( this );
@@ -138,7 +143,7 @@ QgsSimpleLineCalloutWidget::QgsSimpleLineCalloutWidget( QgsVectorLayer *vl, QWid
   mCalloutLineStyleButton->setDialogTitle( tr( "Callout Symbol" ) );
   mCalloutLineStyleButton->registerExpressionContextGenerator( this );
 
-  mCalloutLineStyleButton->setLayer( vl );
+  mCalloutLineStyleButton->setLayer( qobject_cast< QgsVectorLayer * >( vl ) );
   mMinCalloutWidthUnitWidget->setUnits( QgsUnitTypes::RenderUnitList() << Qgis::RenderUnit::Millimeters << Qgis::RenderUnit::MetersInMapUnits << Qgis::RenderUnit::MapUnits << Qgis::RenderUnit::Pixels
                                         << Qgis::RenderUnit::Points << Qgis::RenderUnit::Inches );
   mOffsetFromAnchorUnitWidget->setUnits( QgsUnitTypes::RenderUnitList() << Qgis::RenderUnit::Millimeters << Qgis::RenderUnit::MetersInMapUnits << Qgis::RenderUnit::MapUnits << Qgis::RenderUnit::Pixels
@@ -178,9 +183,11 @@ QgsSimpleLineCalloutWidget::QgsSimpleLineCalloutWidget( QgsVectorLayer *vl, QWid
   connect( mCalloutLineStyleButton, &QgsSymbolButton::changed, this, &QgsSimpleLineCalloutWidget::lineSymbolChanged );
 
   connect( mCalloutBlendComboBox, static_cast<void ( QComboBox::* )( int )>( &QComboBox::currentIndexChanged ), this, &QgsSimpleLineCalloutWidget::mCalloutBlendComboBox_currentIndexChanged );
+
+  mPlacementDDGroupBox->setVisible( qobject_cast< QgsVectorLayer * >( vl ) );
 }
 
-void QgsSimpleLineCalloutWidget::setCallout( QgsCallout *callout )
+void QgsSimpleLineCalloutWidget::setCallout( const QgsCallout *callout )
 {
   if ( !callout )
     return;
@@ -320,7 +327,7 @@ void QgsSimpleLineCalloutWidget::drawToAllPartsToggled( bool active )
 // QgsManhattanLineCalloutWidget
 //
 
-QgsManhattanLineCalloutWidget::QgsManhattanLineCalloutWidget( QgsVectorLayer *vl, QWidget *parent )
+QgsManhattanLineCalloutWidget::QgsManhattanLineCalloutWidget( QgsMapLayer *vl, QWidget *parent )
   : QgsSimpleLineCalloutWidget( vl, parent )
 {
 
@@ -331,7 +338,7 @@ QgsManhattanLineCalloutWidget::QgsManhattanLineCalloutWidget( QgsVectorLayer *vl
 // QgsCurvedLineCalloutWidget
 //
 
-QgsCurvedLineCalloutWidget::QgsCurvedLineCalloutWidget( QgsVectorLayer *vl, QWidget *parent )
+QgsCurvedLineCalloutWidget::QgsCurvedLineCalloutWidget( QgsMapLayer *vl, QWidget *parent )
   : QgsCalloutWidget( parent, vl )
 {
   setupUi( this );
@@ -341,7 +348,7 @@ QgsCurvedLineCalloutWidget::QgsCurvedLineCalloutWidget( QgsVectorLayer *vl, QWid
   mCalloutLineStyleButton->setDialogTitle( tr( "Callout Symbol" ) );
   mCalloutLineStyleButton->registerExpressionContextGenerator( this );
 
-  mCalloutLineStyleButton->setLayer( vl );
+  mCalloutLineStyleButton->setLayer( qobject_cast< QgsVectorLayer * >( vl ) );
   mMinCalloutWidthUnitWidget->setUnits( QgsUnitTypes::RenderUnitList() << Qgis::RenderUnit::Millimeters << Qgis::RenderUnit::MetersInMapUnits << Qgis::RenderUnit::MapUnits << Qgis::RenderUnit::Pixels
                                         << Qgis::RenderUnit::Points << Qgis::RenderUnit::Inches );
   mOffsetFromAnchorUnitWidget->setUnits( QgsUnitTypes::RenderUnitList() << Qgis::RenderUnit::Millimeters << Qgis::RenderUnit::MetersInMapUnits << Qgis::RenderUnit::MapUnits << Qgis::RenderUnit::Pixels
@@ -398,9 +405,11 @@ QgsCurvedLineCalloutWidget::QgsCurvedLineCalloutWidget( QgsVectorLayer *vl, QWid
   } );
 
   connect( mCalloutBlendComboBox, static_cast<void ( QComboBox::* )( int )>( &QComboBox::currentIndexChanged ), this, &QgsCurvedLineCalloutWidget::mCalloutBlendComboBox_currentIndexChanged );
+
+  mPlacementDDGroupBox->setVisible( qobject_cast< QgsVectorLayer * >( vl ) );
 }
 
-void QgsCurvedLineCalloutWidget::setCallout( QgsCallout *callout )
+void QgsCurvedLineCalloutWidget::setCallout( const QgsCallout *callout )
 {
   if ( !callout )
     return;
@@ -547,7 +556,7 @@ void QgsCurvedLineCalloutWidget::drawToAllPartsToggled( bool active )
 // QgsBalloonCalloutWidget
 //
 
-QgsBalloonCalloutWidget::QgsBalloonCalloutWidget( QgsVectorLayer *vl, QWidget *parent )
+QgsBalloonCalloutWidget::QgsBalloonCalloutWidget( QgsMapLayer *vl, QWidget *parent )
   : QgsCalloutWidget( parent, vl )
 {
   setupUi( this );
@@ -557,7 +566,14 @@ QgsBalloonCalloutWidget::QgsBalloonCalloutWidget( QgsVectorLayer *vl, QWidget *p
   mCalloutFillStyleButton->setDialogTitle( tr( "Balloon Symbol" ) );
   mCalloutFillStyleButton->registerExpressionContextGenerator( this );
 
-  mCalloutFillStyleButton->setLayer( vl );
+  mMarkerSymbolButton->setSymbolType( Qgis::SymbolType::Marker );
+  mMarkerSymbolButton->setDialogTitle( tr( "Marker Symbol" ) );
+  mMarkerSymbolButton->registerExpressionContextGenerator( this );
+  mMarkerSymbolButton->setShowNull( true );
+  mMarkerSymbolButton->setToNull();
+
+  mCalloutFillStyleButton->setLayer( qobject_cast< QgsVectorLayer * >( vl ) );
+  mMarkerSymbolButton->setLayer( qobject_cast< QgsVectorLayer * >( vl ) );
   mOffsetFromAnchorUnitWidget->setUnits( QgsUnitTypes::RenderUnitList() << Qgis::RenderUnit::Millimeters << Qgis::RenderUnit::MetersInMapUnits << Qgis::RenderUnit::MapUnits << Qgis::RenderUnit::Pixels
                                          << Qgis::RenderUnit::Points << Qgis::RenderUnit::Inches );
   mMarginUnitWidget->setUnits( QgsUnitTypes::RenderUnitList() << Qgis::RenderUnit::Millimeters << Qgis::RenderUnit::MetersInMapUnits << Qgis::RenderUnit::MapUnits << Qgis::RenderUnit::Pixels
@@ -585,6 +601,7 @@ QgsBalloonCalloutWidget::QgsBalloonCalloutWidget( QgsVectorLayer *vl, QWidget *p
   connect( mAnchorPointComboBox, static_cast<void ( QComboBox::* )( int )>( &QComboBox::currentIndexChanged ), this, &QgsBalloonCalloutWidget::mAnchorPointComboBox_currentIndexChanged );
 
   connect( mCalloutFillStyleButton, &QgsSymbolButton::changed, this, &QgsBalloonCalloutWidget::fillSymbolChanged );
+  connect( mMarkerSymbolButton, &QgsSymbolButton::changed, this, &QgsBalloonCalloutWidget::markerSymbolChanged );
 
   connect( mSpinBottomMargin, qOverload< double >( &QDoubleSpinBox::valueChanged ), this, [ = ]( double value )
   {
@@ -645,9 +662,11 @@ QgsBalloonCalloutWidget::QgsBalloonCalloutWidget( QgsVectorLayer *vl, QWidget *p
   } );
 
   connect( mCalloutBlendComboBox, static_cast<void ( QComboBox::* )( int )>( &QComboBox::currentIndexChanged ), this, &QgsBalloonCalloutWidget::mCalloutBlendComboBox_currentIndexChanged );
+
+  mPlacementDDGroupBox->setVisible( qobject_cast< QgsVectorLayer * >( vl ) );
 }
 
-void QgsBalloonCalloutWidget::setCallout( QgsCallout *callout )
+void QgsBalloonCalloutWidget::setCallout( const QgsCallout *callout )
 {
   if ( !callout )
     return;
@@ -681,6 +700,10 @@ void QgsBalloonCalloutWidget::setCallout( QgsCallout *callout )
   whileBlocking( mCornerRadiusSpin )->setValue( mCallout->cornerRadius() );
 
   whileBlocking( mCalloutFillStyleButton )->setSymbol( mCallout->fillSymbol()->clone() );
+  if ( QgsMarkerSymbol *marker = mCallout->markerSymbol() )
+    whileBlocking( mMarkerSymbolButton )->setSymbol( marker->clone() );
+  else
+    whileBlocking( mMarkerSymbolButton )->setToNull();
 
   whileBlocking( mAnchorPointComboBox )->setCurrentIndex( mAnchorPointComboBox->findData( static_cast< int >( callout->anchorPoint() ) ) );
 
@@ -732,6 +755,12 @@ void QgsBalloonCalloutWidget::fillSymbolChanged()
   emit changed();
 }
 
+void QgsBalloonCalloutWidget::markerSymbolChanged()
+{
+  mCallout->setMarkerSymbol( mMarkerSymbolButton->isNull() ? nullptr : mMarkerSymbolButton->clonedSymbol< QgsMarkerSymbol >() );
+  emit changed();
+}
+
 void QgsBalloonCalloutWidget::mAnchorPointComboBox_currentIndexChanged( int index )
 {
   mCallout->setAnchorPoint( static_cast<QgsCallout::AnchorPoint>( mAnchorPointComboBox->itemData( index ).toInt() ) );
