@@ -120,6 +120,7 @@ void QgsCodeEditorPython::initializeLexer()
   pyLexer->setFont( font, QsciLexerPython::DoubleQuotedString );
 
   pyLexer->setColor( defaultColor, QsciLexerPython::Default );
+  pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::Error ), QsciLexerPython::UnclosedString );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::Class ), QsciLexerPython::ClassName );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::Method ), QsciLexerPython::FunctionMethodName );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::Number ), QsciLexerPython::Number );
@@ -130,7 +131,9 @@ void QgsCodeEditorPython::initializeLexer()
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::Keyword ), QsciLexerPython::Keyword );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::Decoration ), QsciLexerPython::Decorator );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::SingleQuote ), QsciLexerPython::SingleQuotedString );
+  pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::SingleQuote ), QsciLexerPython::SingleQuotedFString );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::DoubleQuote ), QsciLexerPython::DoubleQuotedString );
+  pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::DoubleQuote ), QsciLexerPython::DoubleQuotedFString );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::TripleSingleQuote ), QsciLexerPython::TripleSingleQuotedString );
   pyLexer->setColor( lexerColor( QgsCodeEditorColorScheme::ColorRole::TripleDoubleQuote ), QsciLexerPython::TripleDoubleQuotedString );
 
@@ -233,17 +236,6 @@ void QgsCodeEditorPython::keyPressEvent( QKeyEvent *event )
   bool autoSurround = settings.value( QStringLiteral( "/pythonConsole/autoSurround" ), true ).toBool();
   bool autoInsertImport = settings.value( QStringLiteral( "/pythonConsole/autoInsertImport" ), false ).toBool();
 
-  // Update calltips when cursor position changes with left and right keys
-  if ( event->key() == Qt::Key_Left  ||
-       event->key() == Qt::Key_Right ||
-       event->key() == Qt::Key_Up ||
-       event->key() == Qt::Key_Down )
-  {
-    QgsCodeEditor::keyPressEvent( event );
-    callTip();
-    return;
-  }
-
   // Get entered text and cursor position
   const QString eText = event->text();
   int line, column;
@@ -319,14 +311,13 @@ void QgsCodeEditorPython::keyPressEvent( QKeyEvent *event )
           setSelection( line, column - 1, line, column + 1 );
           removeSelectedText();
           event->accept();
+          // Update calltips (cursor position has changed)
+          callTip();
         }
         else
         {
           QgsCodeEditor::keyPressEvent( event );
         }
-
-        // Update calltips (cursor position has changed)
-        callTip();
         return;
       }
 
