@@ -39,7 +39,7 @@ QgsLocatorWidget::QgsLocatorWidget( QWidget *parent )
 {
   setObjectName( QStringLiteral( "LocatorWidget" ) );
   mLineEdit->setShowClearButton( true );
-#ifdef Q_OS_MACX
+#ifdef Q_OS_MACOS
   mLineEdit->setPlaceholderText( tr( "Type to locate (⌘K)" ) );
 #else
   mLineEdit->setPlaceholderText( tr( "Type to locate (Ctrl+K)" ) );
@@ -83,6 +83,7 @@ QgsLocatorWidget::QgsLocatorWidget( QWidget *parent )
 
   connect( mLineEdit, &QLineEdit::textChanged, this, &QgsLocatorWidget::scheduleDelayedPopup );
   connect( mResultsView, &QAbstractItemView::activated, this, &QgsLocatorWidget::acceptCurrentEntry );
+  connect( mResultsView->selectionModel(), &QItemSelectionModel::selectionChanged, this, &QgsLocatorWidget::selectionChanged );
   connect( mResultsView, &QAbstractItemView::customContextMenuRequested, this, &QgsLocatorWidget::showContextMenu );
 
   connect( mModelBridge, &QgsLocatorModelBridge::resultAdded, this, &QgsLocatorWidget::resultAdded );
@@ -373,6 +374,14 @@ void QgsLocatorWidget::acceptCurrentEntry()
   }
 }
 
+void QgsLocatorWidget::selectionChanged( const QItemSelection &selected, const QItemSelection &deselected )
+{
+  if ( !mResultsView->isVisible() )
+    return;
+
+  mModelBridge->selectionChanged( selected, deselected );
+}
+
 ///@cond PRIVATE
 
 //
