@@ -397,7 +397,7 @@ QList<QgsModelGraphicsScene::LinkSource> QgsModelGraphicsScene::linkSourcesForPa
     for ( const QString &v : list )
       res.append( linkSourcesForParameterValue( model, v, childId, context ) );
   }
-  else if ( value.userType() == QMetaType::type( "QgsProcessingModelChildParameterSource" ) )
+  else if ( value.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>() )
   {
     const QgsProcessingModelChildParameterSource source = value.value< QgsProcessingModelChildParameterSource >();
     switch ( source.source() )
