@@ -220,7 +220,7 @@ QVariant QgsProcessingModelerParameterWidget::value() const
         const QVariantList vList = v.toList();
         if ( std::all_of( vList.begin(), vList.end(), []( const QVariant & val )
       {
-        return val.userType() == QMetaType::type( "QgsProcessingModelChildParameterSource" );
+        return val.userType() == qMetaTypeId<QgsProcessingModelChildParameterSource>();
         } ) )
         {
           return v;
