@@ -21,8 +21,11 @@
 #include <QMessageBox>
 
 #include "qgsproviderregistry.h"
-#include "ogr/qgsogrhelperfunctions.h"
+#include "qgsgdalguiutils.h"
 #include "qgsgdalutils.h"
+#include "qgsgdalcredentialoptionswidget.h"
+#include "qgsspinbox.h"
+#include "qgsdoublespinbox.h"
 
 #include <gdal.h>
 #include <cpl_minixml.h>
@@ -45,17 +48,16 @@ QgsGdalSourceSelect::QgsGdalSourceSelect( QWidget *parent, Qt::WindowFlags fl, Q
   whileBlocking( radioSrcFile )->setChecked( true );
   protocolGroupBox->hide();
 
-  QStringList protocolTypes = QStringLiteral( "HTTP/HTTPS/FTP,vsicurl;AWS S3,vsis3;Google Cloud Storage,vsigs" ).split( ';' );
-  protocolTypes += QStringLiteral( "Microsoft Azure Blob,vsiaz;Microsoft Azure Data Lake Storage,vsiadls;Alibaba Cloud OSS,vsioss;OpenStack Swift Object Storage,vsiswift" ).split( ';' );
-  for ( int i = 0; i < protocolTypes.count(); i++ )
+  QList< QgsGdalUtils::VsiNetworkFileSystemDetails > vsiDetails = QgsGdalUtils::vsiNetworkFileSystems();
+  std::sort( vsiDetails.begin(), vsiDetails.end(), []( const QgsGdalUtils::VsiNetworkFileSystemDetails & a, const QgsGdalUtils::VsiNetworkFileSystemDetails & b )
   {
-    QString protocol = protocolTypes.at( i );
-    if ( ( !protocol.isEmpty() ) && ( !protocol.isNull() ) )
-      cmbProtocolTypes->addItem( protocol.split( ',' ).at( 0 ) );
+    return QString::localeAwareCompare( a.name, b.name ) < 0;
+  } );
+  for ( const QgsGdalUtils::VsiNetworkFileSystemDetails &vsiDetail : std::as_const( vsiDetails ) )
+  {
+    cmbProtocolTypes->addItem( vsiDetail.name, vsiDetail.identifier );
   }
 
-  mAuthWarning->setText( tr( " Additional credential options are required as documented <a href=\"%1\">here</a>." ).arg( QLatin1String( "https://gdal.org/user/virtual_file_systems.html#drivers-supporting-virtual-file-systems" ) ) );
-
   connect( protocolURI, &QLineEdit::textChanged, this, [ = ]( const QString & text )
   {
     if ( radioSrcProtocol->isChecked() )
@@ -91,22 +93,19 @@ QgsGdalSourceSelect::QgsGdalSourceSelect( QWidget *parent, Qt::WindowFlags fl, Q
     fillOpenOptions();
   } );
   mOpenOptionsGroupBox->setVisible( false );
-  mAuthSettingsProtocol->setDataprovider( QStringLiteral( "gdal" ) );
-}
 
-bool QgsGdalSourceSelect::isProtocolCloudType()
-{
-  return ( cmbProtocolTypes->currentText() == QLatin1String( "AWS S3" ) ||
-           cmbProtocolTypes->currentText() == QLatin1String( "Google Cloud Storage" ) ||
-           cmbProtocolTypes->currentText() == QLatin1String( "Microsoft Azure Blob" ) ||
-           cmbProtocolTypes->currentText() == QLatin1String( "Microsoft Azure Data Lake Storage" ) ||
-           cmbProtocolTypes->currentText() == QLatin1String( "Alibaba Cloud OSS" ) ||
-           cmbProtocolTypes->currentText() == QLatin1String( "OpenStack Swift Object Storage" ) );
+  mCredentialsWidget = new QgsGdalCredentialOptionsWidget();
+  mCredentialOptionsLayout->addWidget( mCredentialsWidget );
+  mCredentialOptionsGroupBox->setVisible( false );
+
+  connect( mCredentialsWidget, &QgsGdalCredentialOptionsWidget::optionsChanged, this, &QgsGdalSourceSelect::credentialOptionsChanged );
+
+  mAuthSettingsProtocol->setDataprovider( QStringLiteral( "gdal" ) );
 }
 
 void QgsGdalSourceSelect::setProtocolWidgetsVisibility()
 {
-  if ( isProtocolCloudType() )
+  if ( QgsGdalUtils::vsiHandlerType( cmbProtocolTypes->currentData().toString() ) == Qgis::VsiHandlerType::Cloud )
   {
     labelProtocolURI->hide();
     protocolURI->hide();
@@ -115,7 +114,6 @@ void QgsGdalSourceSelect::setProtocolWidgetsVisibility()
     mBucket->show();
     labelKey->show();
     mKey->show();
-    mAuthWarning->show();
   }
   else
   {
@@ -126,7 +124,6 @@ void QgsGdalSourceSelect::setProtocolWidgetsVisibility()
     mBucket->hide();
     labelKey->hide();
     mKey->hide();
-    mAuthWarning->hide();
   }
 }
 
@@ -137,9 +134,9 @@ void QgsGdalSourceSelect::radioSrcFile_toggled( bool checked )
     fileGroupBox->show();
     protocolGroupBox->hide();
     clearOpenOptions();
+    updateProtocolOptions();
 
     emit enableButtons( !mFileWidget->filePath().isEmpty() );
-
   }
 }
 
@@ -172,6 +169,7 @@ void QgsGdalSourceSelect::radioSrcProtocol_toggled( bool checked )
     protocolGroupBox->show();
     setProtocolWidgetsVisibility();
     clearOpenOptions();
+    updateProtocolOptions();
 
     emit enableButtons( !protocolURI->text().isEmpty() );
   }
@@ -182,6 +180,7 @@ void QgsGdalSourceSelect::cmbProtocolTypes_currentIndexChanged( const QString &t
   Q_UNUSED( text )
   setProtocolWidgetsVisibility();
   clearOpenOptions();
+  updateProtocolOptions();
 }
 
 void QgsGdalSourceSelect::addButtonClicked()
@@ -277,10 +276,32 @@ bool QgsGdalSourceSelect::configureFromUri( const QString &uri )
             cb->setCurrentIndex( idx );
           }
         }
-        else if ( auto le = qobject_cast<QLineEdit *>( *widget ) )
+        else if ( QLineEdit *le = qobject_cast<QLineEdit *>( *widget ) )
         {
           le->setText( opt.value().toString() );
         }
+        else if ( QgsSpinBox *intSpin = qobject_cast<QgsSpinBox *>( *widget ) )
+        {
+          if ( opt.value().toString().isEmpty() )
+          {
+            intSpin->clear();
+          }
+          else
+          {
+            intSpin->setValue( opt.value().toInt() );
+          }
+        }
+        else if ( QgsDoubleSpinBox *doubleSpin = qobject_cast<QgsDoubleSpinBox *>( *widget ) )
+        {
+          if ( opt.value().toString().isEmpty() )
+          {
+            doubleSpin->clear();
+          }
+          else
+          {
+            doubleSpin->setValue( opt.value().toDouble() );
+          }
+        }
       }
     }
   }
@@ -304,12 +325,28 @@ void QgsGdalSourceSelect::computeDataSources()
     {
       value = le->text();
     }
+    else if ( QgsSpinBox *intSpin = qobject_cast<QgsSpinBox *>( control ) )
+    {
+      if ( intSpin->value() != intSpin->clearValue() )
+      {
+        value = QString::number( intSpin->value() );
+      }
+    }
+    else if ( QgsDoubleSpinBox *doubleSpin = qobject_cast<QgsDoubleSpinBox *>( control ) )
+    {
+      if ( doubleSpin->value() != doubleSpin->clearValue() )
+      {
+        value = QString::number( doubleSpin->value() );
+      }
+    }
     if ( !value.isEmpty() )
     {
-      openOptions << QStringLiteral( "%1=%2" ).arg( control->objectName() ).arg( value );
+      openOptions << QStringLiteral( "%1=%2" ).arg( control->objectName(), value );
     }
   }
 
+  const QVariantMap credentialOptions = !mCredentialOptionsGroupBox->isHidden() ? mCredentialOptions : QVariantMap();
+
   if ( radioSrcFile->isChecked() || radioSrcOgcApi->isChecked() )
   {
     for ( const auto &filePath : QgsFileWidget::splitFilePaths( mRasterPath ) )
@@ -323,7 +360,7 @@ void QgsGdalSourceSelect::computeDataSources()
   }
   else if ( radioSrcProtocol->isChecked() )
   {
-    bool cloudType = isProtocolCloudType();
+    const bool cloudType = QgsGdalUtils::vsiHandlerType( cmbProtocolTypes->currentData().toString() ) == Qgis::VsiHandlerType::Cloud;
     if ( !cloudType && protocolURI->text().isEmpty() )
     {
       return;
@@ -346,12 +383,14 @@ void QgsGdalSourceSelect::computeDataSources()
     QVariantMap parts;
     if ( !openOptions.isEmpty() )
       parts.insert( QStringLiteral( "openOptions" ), openOptions );
+    if ( !credentialOptions.isEmpty() )
+      parts.insert( QStringLiteral( "credentialOptions" ), credentialOptions );
     parts.insert( QStringLiteral( "path" ),
-                  createProtocolURI( cmbProtocolTypes->currentText(),
-                                     uri,
-                                     mAuthSettingsProtocol->configId(),
-                                     mAuthSettingsProtocol->username(),
-                                     mAuthSettingsProtocol->password() ) );
+                  QgsGdalGuiUtils::createProtocolURI( cmbProtocolTypes->currentData().toString(),
+                      uri,
+                      mAuthSettingsProtocol->configId(),
+                      mAuthSettingsProtocol->username(),
+                      mAuthSettingsProtocol->password() ) );
     mDataSources << QgsProviderRegistry::instance()->encodeUri( QStringLiteral( "gdal" ), parts );
   }
 }
@@ -388,115 +427,83 @@ void QgsGdalSourceSelect::fillOpenOptions()
     return;
   }
 
+  QVariantMap parts = QgsProviderRegistry::instance()->decodeUri( QStringLiteral( "gdal" ), firstDataSource );
+  const QVariantMap credentialOptions = parts.value( QStringLiteral( "credentialOptions" ) ).toMap();
+  parts.remove( QStringLiteral( "credentialOptions" ) );
+  if ( !credentialOptions.isEmpty() && !vsiPrefix.isEmpty() )
+  {
+    const thread_local QRegularExpression bucketRx( QStringLiteral( "^(.*)/" ) );
+    const QRegularExpressionMatch bucketMatch = bucketRx.match( parts.value( QStringLiteral( "path" ) ).toString() );
+    if ( bucketMatch.hasMatch() )
+    {
+      QgsGdalUtils::applyVsiCredentialOptions( vsiPrefix, bucketMatch.captured( 1 ), credentialOptions );
+    }
+  }
+
+  const QString gdalUri = QgsProviderRegistry::instance()->encodeUri( QStringLiteral( "gdal" ), parts );
   GDALDriverH hDriver;
-  hDriver = GDALIdentifyDriverEx( firstDataSource.toUtf8().toStdString().c_str(), GDAL_OF_RASTER, nullptr, nullptr );
-  if ( hDriver == nullptr )
+  hDriver = GDALIdentifyDriverEx( gdalUri.toUtf8().toStdString().c_str(), GDAL_OF_RASTER, nullptr, nullptr );
+  if ( !hDriver )
     return;
 
   const char *pszOpenOptionList = GDALGetMetadataItem( hDriver, GDAL_DMD_OPENOPTIONLIST, nullptr );
-  if ( pszOpenOptionList == nullptr )
+  if ( !pszOpenOptionList )
     return;
 
   CPLXMLNode *psDoc = CPLParseXMLString( pszOpenOptionList );
-  if ( psDoc == nullptr )
+  if ( !psDoc )
     return;
   CPLXMLNode *psOpenOptionList = CPLGetXMLNode( psDoc, "=OpenOptionList" );
-  if ( psOpenOptionList == nullptr )
+  if ( !psOpenOptionList )
   {
     CPLDestroyXMLNode( psDoc );
     return;
   }
 
-  for ( auto psItem = psOpenOptionList->psChild; psItem != nullptr; psItem = psItem->psNext )
+  const QList< QgsGdalOption > options = QgsGdalOption::optionsFromXml( psOpenOptionList );
+  CPLDestroyXMLNode( psDoc );
+
+  for ( const QgsGdalOption &option : options )
   {
-    if ( psItem->eType != CXT_Element || !EQUAL( psItem->pszValue, "Option" ) )
-      continue;
-
-    const char *pszOptionName = CPLGetXMLValue( psItem, "name", nullptr );
-    if ( pszOptionName == nullptr )
-      continue;
-
     // Exclude options that are not of raster scope
-    const char *pszScope = CPLGetXMLValue( psItem, "scope", nullptr );
-    if ( pszScope != nullptr && strstr( pszScope, "raster" ) == nullptr )
+    if ( !option.scope.isEmpty()
+         && option.scope.compare( QLatin1String( "raster" ), Qt::CaseInsensitive ) != 0 )
       continue;
 
-    const char *pszType = CPLGetXMLValue( psItem, "type", nullptr );
-    QStringList options;
-    if ( pszType && EQUAL( pszType, "string-select" ) )
-    {
-      for ( auto psOption = psItem->psChild; psOption != nullptr; psOption = psOption->psNext )
-      {
-        if ( psOption->eType != CXT_Element ||
-             !EQUAL( psOption->pszValue, "Value" ) ||
-             psOption->psChild == nullptr )
-        {
-          continue;
-        }
-        options << psOption->psChild->pszValue;
-      }
-    }
-
-    QLabel *label = new QLabel( pszOptionName );
-    QWidget *control = nullptr;
-    if ( pszType && EQUAL( pszType, "boolean" ) )
-    {
-      QComboBox *cb = new QComboBox();
-      cb->addItem( tr( "Yes" ), "YES" );
-      cb->addItem( tr( "No" ), "NO" );
-      cb->addItem( tr( "<Default>" ), QVariant( QVariant::String ) );
-      int idx = cb->findData( QVariant( QVariant::String ) );
-      cb->setCurrentIndex( idx );
-      control = cb;
-    }
-    else if ( !options.isEmpty() )
-    {
-      QComboBox *cb = new QComboBox();
-      for ( const QString &val : std::as_const( options ) )
-      {
-        cb->addItem( val, val );
-      }
-      cb->addItem( tr( "<Default>" ), QVariant( QVariant::String ) );
-      int idx = cb->findData( QVariant( QVariant::String ) );
+    QWidget *control = QgsGdalGuiUtils::createWidgetForOption( option, nullptr, true );
+    if ( !control )
+      continue;
 
 #if GDAL_VERSION_NUM >= GDAL_COMPUTE_VERSION(3,8,0)
-      if ( QString( GDALGetDriverShortName( hDriver ) ).compare( QLatin1String( "BAG" ) ) == 0 && label->text() == QLatin1String( "MODE" ) && options.contains( QLatin1String( "INTERPOLATED" ) ) )
+    if ( QString( GDALGetDriverShortName( hDriver ) ).compare( QLatin1String( "BAG" ) ) == 0
+         && option.name == QLatin1String( "MODE" ) && option.options.contains( QLatin1String( "INTERPOLATED" ) ) )
+    {
+      gdal::dataset_unique_ptr hSrcDS( GDALOpen( gdalUri.toUtf8().constData(), GA_ReadOnly ) );
+      if ( hSrcDS && QString{ GDALGetMetadataItem( hSrcDS.get(), "HAS_SUPERGRIDS", nullptr ) } == QLatin1String( "TRUE" ) )
       {
-        gdal::dataset_unique_ptr hSrcDS( GDALOpen( firstDataSource.toUtf8().constData(), GA_ReadOnly ) );
-        if ( hSrcDS && QString{ GDALGetMetadataItem( hSrcDS.get(), "HAS_SUPERGRIDS", nullptr ) } == QLatin1String( "TRUE" ) )
+        if ( QComboBox *combo = qobject_cast< QComboBox * >( control ) )
         {
-          idx = cb->findText( QLatin1String( "INTERPOLATED" ) );
+          combo->setCurrentIndex( combo->findText( QLatin1String( "INTERPOLATED" ) ) );
         }
       }
+    }
 #endif
 
-      cb->setCurrentIndex( idx );
-      control = cb;
-    }
-    else
-    {
-      QLineEdit *le = new QLineEdit( );
-      control = le;
-    }
-    control->setObjectName( pszOptionName );
+    control->setObjectName( option.name );
     mOpenOptionsWidgets.push_back( control );
 
-    const char *pszDescription = CPLGetXMLValue( psItem, "description", nullptr );
-    if ( pszDescription )
-    {
-      label->setToolTip( QStringLiteral( "<p>%1</p>" ).arg( pszDescription ) );
-      control->setToolTip( QStringLiteral( "<p>%1</p>" ).arg( pszDescription ) );
-    }
+    QLabel *label = new QLabel( option.name );
+    if ( !option.description.isEmpty() )
+      label->setToolTip( QStringLiteral( "<p>%1</p>" ).arg( option.description ) );
+
     mOpenOptionsLayout->addRow( label, control );
   }
 
-  CPLDestroyXMLNode( psDoc );
-
   // Set label to point to driver help page
-  const char *pszHelpTopic = GDALGetMetadataItem( hDriver, GDAL_DMD_HELPTOPIC, nullptr );
-  if ( pszHelpTopic )
+  const QString helpTopic = QgsGdalUtils::gdalDocumentationUrlForDriver( hDriver );
+  if ( !helpTopic.isEmpty() )
   {
-    mOpenOptionsLabel->setText( tr( "Consult <a href=\"https://gdal.org/%1\">%2 driver help page</a> for detailed explanations on options" ).arg( pszHelpTopic ).arg( GDALGetDriverShortName( hDriver ) ) );
+    mOpenOptionsLabel->setText( tr( "Consult <a href=\"%1\">%2 driver help page</a> for detailed explanations on options" ).arg( helpTopic ).arg( GDALGetDriverShortName( hDriver ) ) );
     mOpenOptionsLabel->setTextInteractionFlags( Qt::TextBrowserInteraction );
     mOpenOptionsLabel->setOpenExternalLinks( true );
     mOpenOptionsLabel->setVisible( true );
@@ -507,7 +514,6 @@ void QgsGdalSourceSelect::fillOpenOptions()
   }
 
   mOpenOptionsGroupBox->setVisible( !mOpenOptionsWidgets.empty() );
-
 }
 
 void QgsGdalSourceSelect::showHelp()
@@ -515,4 +521,28 @@ void QgsGdalSourceSelect::showHelp()
   QgsHelp::openHelp( QStringLiteral( "managing_data_source/opening_data.html#loading-a-layer-from-a-file" ) );
 }
 
+void QgsGdalSourceSelect::updateProtocolOptions()
+{
+  const QString currentProtocol = cmbProtocolTypes->currentData().toString();
+  if ( radioSrcProtocol->isChecked() && QgsGdalUtils::vsiHandlerType( currentProtocol ) == Qgis::VsiHandlerType::Cloud )
+  {
+    mCredentialsWidget->setHandler( currentProtocol );
+    mCredentialOptionsGroupBox->setVisible( true );
+  }
+  else
+  {
+    mCredentialOptionsGroupBox->setVisible( false );
+  }
+}
+
+void QgsGdalSourceSelect::credentialOptionsChanged()
+{
+  const QVariantMap newCredentialOptions = mCredentialsWidget->credentialOptions();
+  if ( newCredentialOptions == mCredentialOptions )
+    return;
+
+  mCredentialOptions = newCredentialOptions;
+  fillOpenOptions();
+}
+
 ///@endcond
