@@ -169,6 +169,13 @@ QgsNewVectorTableDialog::QgsNewVectorTableDialog( QgsAbstractDatabaseProviderCon
     addGeomItem( Qgis::WkbType::MultiSurface );
   }
 
+  if ( conn->geometryColumnCapabilities().testFlag( QgsAbstractDatabaseProviderConnection::GeometryColumnCapability::PolyhedralSurfaces ) )
+  {
+    addGeomItem( Qgis::WkbType::Triangle ); // Not exactly surfaces, but only PostGIS (and memory) supports these types
+    addGeomItem( Qgis::WkbType::PolyhedralSurface );
+    addGeomItem( Qgis::WkbType::TIN );
+  }
+
   mGeomTypeCbo->setCurrentIndex( 0 );
 
   const bool hasZ { conn->geometryColumnCapabilities().testFlag( QgsAbstractDatabaseProviderConnection::GeometryColumnCapability::Z ) };
