@@ -30,7 +30,7 @@
 #include "qgsvariantutils.h"
 
 
-QgsAfsProvider::QgsAfsProvider( const QString &uri, const ProviderOptions &options, QgsDataProvider::ReadFlags flags )
+QgsAfsProvider::QgsAfsProvider( const QString &uri, const ProviderOptions &options, Qgis::DataProviderReadFlags flags )
   : QgsVectorDataProvider( uri, options, flags )
 {
   mSharedData.reset( new QgsAfsSharedData( QgsDataSourceUri( uri ) ) );
@@ -313,6 +313,11 @@ QgsAfsProvider::QgsAfsProvider( const QString &uri, const ProviderOptions &optio
   // renderer
   mRendererDataMap = layerData.value( QStringLiteral( "drawingInfo" ) ).toMap().value( QStringLiteral( "renderer" ) ).toMap();
   mLabelingDataList = layerData.value( QStringLiteral( "drawingInfo" ) ).toMap().value( QStringLiteral( "labelingInfo" ) ).toList();
+  const QVariant transparency = layerData.value( QStringLiteral( "drawingInfo" ) ).toMap().value( QStringLiteral( "transparency" ) );
+  if ( transparency.isValid() )
+  {
+    mRendererDataMap.insert( QStringLiteral( "transparency" ), transparency );
+  }
 
   mValid = true;
 }
@@ -581,46 +586,46 @@ bool QgsAfsProvider::createAttributeIndex( int field )
   return true;
 }
 
-QgsVectorDataProvider::Capabilities QgsAfsProvider::capabilities() const
+Qgis::VectorProviderCapabilities QgsAfsProvider::capabilities() const
 {
-  QgsVectorDataProvider::Capabilities c = QgsVectorDataProvider::SelectAtId
-                                          | QgsVectorDataProvider::ReadLayerMetadata
-                                          | QgsVectorDataProvider::Capability::ReloadData;
+  Qgis::VectorProviderCapabilities c = Qgis::VectorProviderCapability::SelectAtId
+                                       | Qgis::VectorProviderCapability::ReadLayerMetadata
+                                       | Qgis::VectorProviderCapability::ReloadData;
   if ( !mRendererDataMap.empty() )
   {
-    c = c | QgsVectorDataProvider::CreateRenderer;
+    c = c | Qgis::VectorProviderCapability::CreateRenderer;
   }
   if ( !mLabelingDataList.empty() )
   {
-    c = c | QgsVectorDataProvider::CreateLabeling;
+    c = c | Qgis::VectorProviderCapability::CreateLabeling;
   }
 
   if ( mServerSupportsCurves )
-    c |= QgsVectorDataProvider::CircularGeometries;
+    c |= Qgis::VectorProviderCapability::CircularGeometries;
 
   if ( mCapabilityStrings.contains( QLatin1String( "delete" ), Qt::CaseInsensitive ) )
   {
-    c |= QgsVectorDataProvider::DeleteFeatures;
+    c |= Qgis::VectorProviderCapability::DeleteFeatures;
   }
   if ( mCapabilityStrings.contains( QLatin1String( "create" ), Qt::CaseInsensitive ) )
   {
-    c |= QgsVectorDataProvider::AddFeatures;
+    c |= Qgis::VectorProviderCapability::AddFeatures;
   }
   if ( mCapabilityStrings.contains( QLatin1String( "update" ), Qt::CaseInsensitive ) )
   {
-    c |= QgsVectorDataProvider::ChangeAttributeValues;
-    c |= QgsVectorDataProvider::ChangeFeatures;
-    c |= QgsVectorDataProvider::ChangeGeometries;
+    c |= Qgis::VectorProviderCapability::ChangeAttributeValues;
+    c |= Qgis::VectorProviderCapability::ChangeFeatures;
+    c |= Qgis::VectorProviderCapability::ChangeGeometries;
   }
 
   if ( mAdminCapabilityStrings.contains( QLatin1String( "update" ), Qt::CaseInsensitive ) )
   {
-    c |= QgsVectorDataProvider::AddAttributes;
-    c |= QgsVectorDataProvider::CreateAttributeIndex;
+    c |= Qgis::VectorProviderCapability::AddAttributes;
+    c |= Qgis::VectorProviderCapability::CreateAttributeIndex;
   }
   if ( mAdminCapabilityStrings.contains( QLatin1String( "delete" ), Qt::CaseInsensitive ) )
   {
-    c |= QgsVectorDataProvider::DeleteAttributes;
+    c |= Qgis::VectorProviderCapability::DeleteAttributes;
   }
 
   return c;
@@ -673,6 +678,21 @@ bool QgsAfsProvider::setSubsetString( const QString &subset, bool )
   return true;
 }
 
+bool QgsAfsProvider::supportsSubsetString() const
+{
+  return true;
+}
+
+QString QgsAfsProvider::subsetStringDialect() const
+{
+  return tr( "SQL 92 WHERE clause" );
+}
+
+QString QgsAfsProvider::subsetStringHelpUrl() const
+{
+  return QStringLiteral( "https://developers.arcgis.com/rest/services-reference/enterprise/query-feature-service/#sql-92-where-clause" );
+}
+
 void QgsAfsProvider::setDataSourceUri( const QString &uri )
 {
   mSharedData->mDataSource = QgsDataSourceUri( uri );
@@ -797,7 +817,7 @@ QString QgsAfsProviderMetadata::encodeUri( const QVariantMap &parts ) const
   QgsDataSourceUri dsUri;
   dsUri.setParam( QStringLiteral( "url" ), parts.value( QStringLiteral( "url" ) ).toString() );
 
-  if ( parts.contains( QStringLiteral( "bounds" ) ) && parts.value( QStringLiteral( "bounds" ) ).userType() == QMetaType::type( "QgsRectangle" ) )
+  if ( parts.contains( QStringLiteral( "bounds" ) ) && parts.value( QStringLiteral( "bounds" ) ).userType() == qMetaTypeId<QgsRectangle>() )
   {
     const QgsRectangle bBox = parts.value( QStringLiteral( "bounds" ) ).value< QgsRectangle >();
     dsUri.setParam( QStringLiteral( "bbox" ), QStringLiteral( "%1,%2,%3,%4" ).arg( bBox.xMinimum() ).arg( bBox.yMinimum() ).arg( bBox.xMaximum() ).arg( bBox.yMaximum() ) );
@@ -817,7 +837,7 @@ QString QgsAfsProviderMetadata::encodeUri( const QVariantMap &parts ) const
   return dsUri.uri( false );
 }
 
-QgsAfsProvider *QgsAfsProviderMetadata::createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags )
+QgsAfsProvider *QgsAfsProviderMetadata::createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options, Qgis::DataProviderReadFlags flags )
 {
   return new QgsAfsProvider( uri, options, flags );
 }
