@@ -25,6 +25,7 @@
 #include "qgsgeometry.h"
 #include "qgsapplication.h"
 #include "qgsnetworkaccessmanager.h"
+#include "qgssetrequestinitiator_p.h"
 #include "qgssettings.h"
 #include "qgsmessagelog.h"
 #include "qgsauthmanager.h"
@@ -186,7 +187,7 @@ void QgsAmsLegendFetcher::handleFinished()
 
 ///////////////////////////////////////////////////////////////////////////////
 
-QgsAmsProvider::QgsAmsProvider( const QString &uri, const ProviderOptions &options, QgsDataProvider::ReadFlags flags )
+QgsAmsProvider::QgsAmsProvider( const QString &uri, const ProviderOptions &options, Qgis::DataProviderReadFlags flags )
   : QgsRasterDataProvider( uri, options, flags )
 {
   QgsDataSourceUri dataSource( dataSourceUri() );
@@ -346,15 +347,28 @@ QgsAmsProvider::QgsAmsProvider( const QgsAmsProvider &other, const QgsDataProvid
   mTimestamp = QDateTime::currentDateTime();
 }
 
-QgsRasterDataProvider::ProviderCapabilities QgsAmsProvider::providerCapabilities() const
+Qgis::DataProviderFlags QgsAmsProvider::flags() const
 {
-  return ProviderCapability::ReadLayerMetadata | ProviderCapability::ReloadData;
+  return Qgis::DataProviderFlag::FastExtent2D;
+}
+
+Qgis::RasterProviderCapabilities QgsAmsProvider::providerCapabilities() const
+{
+  return Qgis::RasterProviderCapability::ReadLayerMetadata | Qgis::RasterProviderCapability::ReloadData;
 }
 
 QString QgsAmsProvider::name() const { return AMS_PROVIDER_KEY; }
 
 QString QgsAmsProvider::providerKey() { return AMS_PROVIDER_KEY; }
 
+Qgis::RasterInterfaceCapabilities QgsAmsProvider::capabilities() const
+{
+  return Qgis::RasterInterfaceCapability::Identify
+         | Qgis::RasterInterfaceCapability::IdentifyText
+         | Qgis::RasterInterfaceCapability::IdentifyFeature
+         | Qgis::RasterInterfaceCapability::Prefetch;
+}
+
 QString QgsAmsProvider::description() const { return AMS_PROVIDER_DESCRIPTION; }
 
 QStringList QgsAmsProvider::subLayerStyles() const
@@ -884,7 +898,7 @@ QgsRasterIdentifyResult QgsAmsProvider::identify( const QgsPointXY &point, Qgis:
       QgsAttributes featureAttributes;
       for ( auto it = attributesMap.constBegin(); it != attributesMap.constEnd(); ++it )
       {
-        fields.append( QgsField( it.key(), QVariant::String, QStringLiteral( "string" ) ) );
+        fields.append( QgsField( it.key(), QMetaType::Type::QString, QStringLiteral( "string" ) ) );
         featureAttributes.append( it.value().toString() );
       }
       QgsCoordinateReferenceSystem crs;
@@ -1259,7 +1273,7 @@ QIcon QgsAmsProviderMetadata::icon() const
   return QgsApplication::getThemeIcon( QStringLiteral( "mIconAms.svg" ) );
 }
 
-QgsAmsProvider *QgsAmsProviderMetadata::createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags )
+QgsAmsProvider *QgsAmsProviderMetadata::createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options, Qgis::DataProviderReadFlags flags )
 {
   return new QgsAmsProvider( uri, options, flags );
 }
