@@ -133,10 +133,10 @@ void TestQgsVectorTileLayer::test_basic()
 {
   // tile fetch test
   const QgsVectorTileRawData tile0rawData = mLayer->getRawTile( QgsTileXYZ( 0, 0, 0 ) );
-  QCOMPARE( tile0rawData.data.length(), 64822 );
+  QCOMPARE( tile0rawData.data.first().length(), 64822 );
 
   const QgsVectorTileRawData invalidTileRawData = mLayer->getRawTile( QgsTileXYZ( 0, 0, 99 ) );
-  QCOMPARE( invalidTileRawData.data.length(), 0 );
+  QCOMPARE( invalidTileRawData.data.first().length(), 0 );
 
   // an xyz vector tile layer should be considered as a basemap layer
   QCOMPARE( mLayer->properties(), Qgis::MapLayerProperties( Qgis::MapLayerProperty::IsBasemapLayer ) );
