@@ -11,12 +11,14 @@ __copyright__ = 'Copyright 2019, The QGIS Project'
 
 from qgis.PyQt.QtCore import (
     QCoreApplication,
+    QDir,
     QEvent,
     QModelIndex,
     Qt,
     QTemporaryDir,
+    QTemporaryFile
 )
-from qgis.PyQt.QtGui import QColor, QFont
+from qgis.PyQt.QtGui import QColor, QColorSpace, QFont
 from qgis.PyQt.QtTest import QSignalSpy
 from qgis.PyQt.QtXml import QDomDocument
 from qgis.core import (
@@ -32,7 +34,9 @@ from qgis.core import (
     QgsTextFormat,
     QgsWkbTypes,
 )
+
 import unittest
+import os
 from qgis.testing import start_app, QgisTestCase
 
 from utilities import unitTestDataPath
@@ -49,7 +53,9 @@ TEST_DATA_DIR = unitTestDataPath()
 class TestQgsProjectViewSettings(QgisTestCase):
 
     def testDefaultSymbol(self):
-        p = QgsProjectStyleSettings()
+        project = QgsProject()
+        p = project.styleSettings()
+        self.assertFalse(project.isDirty())
         self.assertFalse(p.defaultSymbol(Qgis.SymbolType.Marker))
         self.assertFalse(p.defaultSymbol(Qgis.SymbolType.Line))
         self.assertFalse(p.defaultSymbol(Qgis.SymbolType.Fill))
@@ -57,43 +63,60 @@ class TestQgsProjectViewSettings(QgisTestCase):
         marker = QgsSymbol.defaultSymbol(QgsWkbTypes.GeometryType.PointGeometry)
         p.setDefaultSymbol(Qgis.SymbolType.Marker, marker)
         self.assertTrue(p.defaultSymbol(Qgis.SymbolType.Marker))
+        self.assertTrue(project.isDirty())
+        project.setDirty(False)
 
         line = QgsSymbol.defaultSymbol(QgsWkbTypes.GeometryType.LineGeometry)
         p.setDefaultSymbol(Qgis.SymbolType.Line, line)
+        self.assertTrue(project.isDirty())
         self.assertTrue(p.defaultSymbol(Qgis.SymbolType.Line))
+        project.setDirty(False)
 
         fill = QgsSymbol.defaultSymbol(QgsWkbTypes.GeometryType.PolygonGeometry)
         p.setDefaultSymbol(Qgis.SymbolType.Fill, fill)
+        self.assertTrue(project.isDirty())
         self.assertTrue(p.defaultSymbol(Qgis.SymbolType.Fill))
 
     def testDefaultColorRamp(self):
-        p = QgsProjectStyleSettings()
+        project = QgsProject()
+        p = project.styleSettings()
+        self.assertFalse(project.isDirty())
         self.assertFalse(p.defaultColorRamp())
 
         ramp = QgsGradientColorRamp(QColor(255, 255, 255), QColor(255, 0, 0))
         p.setDefaultColorRamp(ramp)
         self.assertTrue(p.defaultColorRamp())
+        self.assertTrue(project.isDirty())
 
     def testDefaultTextFormat(self):
-        p = QgsProjectStyleSettings()
+        project = QgsProject()
+        p = project.styleSettings()
+        self.assertFalse(project.isDirty())
         self.assertFalse(p.defaultTextFormat().isValid())
 
         textFormat = QgsTextFormat()
         textFormat.setFont(QFont())
         p.setDefaultTextFormat(textFormat)
         self.assertTrue(p.defaultTextFormat().isValid())
+        self.assertTrue(project.isDirty())
 
     def testRandomizeDefaultSymbolColor(self):
-        p = QgsProjectStyleSettings()
+        project = QgsProject()
+        p = project.styleSettings()
+        self.assertFalse(project.isDirty())
         self.assertTrue(p.randomizeDefaultSymbolColor())
         p.setRandomizeDefaultSymbolColor(False)
         self.assertFalse(p.randomizeDefaultSymbolColor())
+        self.assertTrue(project.isDirty())
 
     def testDefaultSymbolOpacity(self):
-        p = QgsProjectStyleSettings()
+        project = QgsProject()
+        p = project.styleSettings()
+        self.assertFalse(project.isDirty())
         self.assertEqual(p.defaultSymbolOpacity(), 1.0)
         p.setDefaultSymbolOpacity(0.25)
         self.assertEqual(p.defaultSymbolOpacity(), 0.25)
+        self.assertTrue(project.isDirty())
 
     def testProjectStyle(self):
         project = QgsProject()
@@ -409,7 +432,8 @@ class TestQgsProjectViewSettings(QgisTestCase):
             QgsStyle.defaultStyle())
 
     def testReadWrite(self):
-        p = QgsProjectStyleSettings()
+        project = QgsProject()
+        p = project.styleSettings()
 
         line = QgsSymbol.defaultSymbol(QgsWkbTypes.GeometryType.LineGeometry)
         p.setDefaultSymbol(Qgis.SymbolType.Line, line)
@@ -443,6 +467,64 @@ class TestQgsProjectViewSettings(QgisTestCase):
         self.assertEqual(p2.styleDatabasePaths(),
                          [unitTestDataPath() + '/style1.db', unitTestDataPath() + '/style2.db'])
 
+    def testColorSettings(self):
+        """
+        Test ICC profile attachment
+        """
+        project = QgsProject()
+        settings = project.styleSettings()
+        self.assertFalse(project.isDirty())
+
+        self.assertEqual(settings.colorModel(), Qgis.ColorModel.Rgb)
+        self.assertFalse(settings.colorSpace().isValid())
+
+        # set Cmyk color model and color space
+
+        settings.setColorModel(Qgis.ColorModel.Cmyk)
+        self.assertTrue(project.isDirty())
+        project.setDirty(False)
+        self.assertEqual(settings.colorModel(), Qgis.ColorModel.Cmyk)
+
+        with open(os.path.join(TEST_DATA_DIR, "sRGB2014.icc"), mode='rb') as f:
+            colorSpace = QColorSpace.fromIccProfile(f.read())
+
+        self.assertTrue(colorSpace.isValid())
+
+        settings.setColorSpace(colorSpace)
+        self.assertTrue(project.isDirty())
+        self.assertTrue(settings.colorSpace().isValid())
+        self.assertEqual(settings.colorSpace().primaries(), QColorSpace.Primaries.SRgb)
+        self.assertEqual(len(project.attachedFiles()), 2)
+
+        # save and restore
+        projectFile = QTemporaryFile(QDir.temp().absoluteFilePath("testCmykSettings.qgz"))
+        projectFile.open()
+        self.assertTrue(project.write(projectFile.fileName()))
+
+        project = QgsProject()
+        self.assertTrue(project.read(projectFile.fileName()))
+        settings = project.styleSettings()
+        self.assertEqual(settings.colorModel(), Qgis.ColorModel.Cmyk)
+        self.assertTrue(settings.colorSpace().isValid())
+        self.assertEqual(settings.colorSpace().primaries(), QColorSpace.Primaries.SRgb)
+        self.assertEqual(len(project.attachedFiles()), 2)
+
+        # clear color space
+        settings.setColorSpace(QColorSpace())
+        self.assertFalse(settings.colorSpace().isValid())
+        self.assertEqual(len(project.attachedFiles()), 1)
+
+        # save and restore cleared
+        projectFile = QTemporaryFile(QDir.temp().absoluteFilePath("testCmykSettingsCleared.qgz"))
+        projectFile.open()
+        self.assertTrue(project.write(projectFile.fileName()))
+
+        project = QgsProject()
+        self.assertTrue(project.read(projectFile.fileName()))
+        settings = project.styleSettings()
+        self.assertFalse(settings.colorSpace().isValid())
+        self.assertEqual(len(project.attachedFiles()), 1)
+
 
 if __name__ == '__main__':
     unittest.main()
